cross_universe = {
    "instrumentType": "FxCross",
    "instrumentDefinition": {
        "legs": [
            {
                "tenor": "4Y",
                "dealCcyBuySell": "Buy",
                "contraAmount": 65762500,
                "dealAmount": 1000000,
            }
        ],
        "fxCrossType": "FxNonDeliverableForward",
        "fxCrossCode": "USDINR",
    },
    "pricingParameters": {"valuationDate": "2017-11-15T00:00:00Z"},
}
