from ...._curves._cross_currency_curves._enums import (
    InterpolationMode,
    MainConstituentAssetClass,
    RiskType,
)
from ...._curves._cross_currency_curves._definitions._enums import (
    QuotationMode,
    StandardTurnPeriod,
)
