from enum import Enum, auto


class ContentType(Enum):
    CHAINS = auto()
    CONTRACTS = auto()
    CUSTOM_INSTRUMENTS_EVENTS = auto()
    CUSTOM_INSTRUMENTS_INSTRUMENTS = auto()
    CUSTOM_INSTRUMENTS_INTERDAY_SUMMARIES = auto()
    CUSTOM_INSTRUMENTS_INTRADAY_SUMMARIES = auto()
    CUSTOM_INSTRUMENTS_SEARCH = auto()
    DATA_GRID_RDP = auto()
    DATA_GRID_UDF = auto()
    DEFAULT = auto()
    DISCOVERY_LOOKUP = auto()
    DISCOVERY_METADATA = auto()
    DISCOVERY_SEARCH = auto()
    ESG_BASIC_OVERVIEW = auto()
    ESG_FULL_MEASURES = auto()
    ESG_FULL_SCORES = auto()
    ESG_STANDARD_MEASURES = auto()
    ESG_STANDARD_SCORES = auto()
    ESG_UNIVERSE = auto()
    ESTIMATES_VIEW_ACTUALS_ANNUAL = auto()
    ESTIMATES_VIEW_ACTUALS_INTERIM = auto()
    ESTIMATES_VIEW_ACTUALS_KPI_ANNUAL = auto()
    ESTIMATES_VIEW_ACTUALS_KPI_INTERIM = auto()
    ESTIMATES_VIEW_SUMMARY_ANNUAL = auto()
    ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_NON_PERIODIC_MEASURES = auto()
    ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_PERIODIC_MEASURES_ANNUAL = auto()
    ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_PERIODIC_MEASURES_INTERIM = auto()
    ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_RECOMMENDATIONS = auto()
    ESTIMATES_VIEW_SUMMARY_INTERIM = auto()
    ESTIMATES_VIEW_SUMMARY_KPI_ANNUAL = auto()
    ESTIMATES_VIEW_SUMMARY_KPI_HISTORICAL_SNAPSHOTS_KPI = auto()
    ESTIMATES_VIEW_SUMMARY_KPI_INTERIM = auto()
    ESTIMATES_VIEW_SUMMARY_NON_PERIODIC_MEASURES = auto()
    ESTIMATES_VIEW_SUMMARY_RECOMMENDATIONS = auto()
    FILINGS_RETRIEVAL = auto()
    FILINGS_SEARCH = auto()
    BOND_CURVE = auto()
    FORWARD_CURVE = auto()
    CROSS_CURRENCY_CURVES_CURVES = auto()
    CROSS_CURRENCY_CURVES_DEFINITIONS_CREATE = auto()
    CROSS_CURRENCY_CURVES_DEFINITIONS_DELETE = auto()
    CROSS_CURRENCY_CURVES_DEFINITIONS_GET = auto()
    CROSS_CURRENCY_CURVES_DEFINITIONS_UPDATE = auto()
    CROSS_CURRENCY_CURVES_DEFINITIONS_SEARCH = auto()
    CROSS_CURRENCY_CURVES_TRIANGULATE_DEFINITIONS_SEARCH = auto()
    HISTORICAL_PRICING_EVENTS = auto()
    HISTORICAL_PRICING_INTERDAY_SUMMARIES = auto()
    HISTORICAL_PRICING_INTRADAY_SUMMARIES = auto()
    NEWS_HEADLINES_RDP = auto()
    NEWS_HEADLINES_UDF = auto()
    NEWS_STORY_RDP = auto()
    NEWS_IMAGES = auto()
    NEWS_STORY_UDF = auto()
    NEWS_TOP_NEWS = auto()
    NEWS_TOP_NEWS_HEADLINES = auto()
    NONE = auto()
    OWNERSHIP_CONSOLIDATED_BREAKDOWN = auto()
    OWNERSHIP_CONSOLIDATED_CONCENTRATION = auto()
    OWNERSHIP_CONSOLIDATED_INVESTORS = auto()
    OWNERSHIP_CONSOLIDATED_RECENT_ACTIVITY = auto()
    OWNERSHIP_CONSOLIDATED_SHAREHOLDERS_HISTORY_REPORT = auto()
    OWNERSHIP_CONSOLIDATED_SHAREHOLDERS_REPORT = auto()
    OWNERSHIP_CONSOLIDATED_TOP_N_CONCENTRATION = auto()
    OWNERSHIP_FUND_BREAKDOWN = auto()
    OWNERSHIP_FUND_CONCENTRATION = auto()
    OWNERSHIP_FUND_HOLDINGS = auto()
    OWNERSHIP_FUND_INVESTORS = auto()
    OWNERSHIP_FUND_RECENT_ACTIVITY = auto()
    OWNERSHIP_FUND_SHAREHOLDERS_HISTORY_REPORT = auto()
    OWNERSHIP_FUND_SHAREHOLDERS_REPORT = auto()
    OWNERSHIP_FUND_TOP_N_CONCENTRATION = auto()
    OWNERSHIP_INSIDER_SHAREHOLDERS_REPORT = auto()
    OWNERSHIP_INSIDER_TRANSACTION_REPORT = auto()
    OWNERSHIP_INVESTOR_HOLDINGS = auto()
    OWNERSHIP_ORG_INFO = auto()
    PRICING = auto()
    STREAMING_CHAINS = auto()
    STREAMING_CONTRACTS = auto()
    STREAMING_CUSTOM = auto()
    STREAMING_CUSTOM_INSTRUMENTS = auto()
    STREAMING_PRICING = auto()
    STREAMING_TRADING = auto()
    STREAMING_CONTRIB = auto()
    STREAMING_OFF_CONTRIB = auto()
    SURFACES = auto()
    ZC_CURVE_DEFINITIONS = auto()
    ZC_CURVES = auto()
    DATES_AND_CALENDARS_ADD_PERIODS = auto()
    DATES_AND_CALENDARS_HOLIDAYS = auto()
    DATES_AND_CALENDARS_COUNT_PERIODS = auto()
    DATES_AND_CALENDARS_DATE_SCHEDULE = auto()
    DATES_AND_CALENDARS_IS_WORKING_DAY = auto()
