from enum import Enum, unique


@unique
class Rating(Enum):
    A = "A"
    A1 = "A1"
    A2 = "A2"
    A3 = "A3"
    AA = "AA"
    AAA = "AAA"
    AA_HIGH = "AAHigh"
    AA_LOW = "AALow"
    AA_MINUS = "AAMinus"
    AA_PLUS = "AAPlus"
    A_HIGH = "AHigh"
    A_LOW = "ALow"
    A_MINUS = "AMinus"
    A_PLUS = "APlus"
    AA1 = "Aa1"
    AA2 = "Aa2"
    AA3 = "Aa3"
    A_AA = "Aaa"
    B = "B"
    B1 = "B1"
    B2 = "B2"
    B3 = "B3"
    BB = "BB"
    BBB = "BBB"
    BBB_HIGH = "BBBHigh"
    BBB_LOW = "BBBLow"
    BBB_MINUS = "BBBMinus"
    BBB_PLUS = "BBBPlus"
    BB_HIGH = "BBHigh"
    BB_LOW = "BBLow"
    BB_MINUS = "BBMinus"
    BB_PLUS = "BBPlus"
    B_HIGH = "BHigh"
    B_LOW = "BLow"
    B_MINUS = "BMinus"
    B_PLUS = "BPlus"
    BA1 = "Ba1"
    BA2 = "Ba2"
    BA3 = "Ba3"
    BAA1 = "Baa1"
    BAA2 = "Baa2"
    BAA3 = "Baa3"
    C = "C"
    CC = "CC"
    CCC = "CCC"
    CCC_HIGH = "CCCHigh"
    CCC_LOW = "CCCLow"
    CCC_MINUS = "CCCMinus"
    CCC_PLUS = "CCCPlus"
    CC_HIGH = "CCHigh"
    CC_LOW = "CCLow"
    C_HIGH = "CHigh"
    C_LOW = "CLow"
    CA = "Ca"
    CAA1 = "Caa1"
    CAA2 = "Caa2"
    CAA3 = "Caa3"
    D = "D"
    DD = "DD"
    DDD = "DDD"
    F1 = "F1"
    F1_PLUS = "F1Plus"
    F2 = "F2"
    F3 = "F3"
