zc_curves_universe = {
    "curveDefinition": {
        "currency": "CHF",
        "discountingTenor": "OIS",
        "name": "CHF LIBOR Swap ZC Curve",
    },
    "curveParameters": {"useSteps": True, "valuationDate": "2019-08-21"},
}
