top_news_headlines_df = """                                                                                                   headline                                                                                                                                                                                                                                                                                        snippet                                     storyId                                               imageId
versionCreated                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
2022-11-22 11:56:00                                       Europe to be hit hardest in global slowdown -OECD                                                  The global economy should avoid a recession next year but the worst energy crisis since the 1970s will trigger a sharp slowdown, with Europe hit hardest, the OECD said, adding that fighting inflation should be policymakers' top priority.  urn:newsml:reuters.com:20221122:nL1N32I0MO  2022-11-22T100711Z_1_AT0_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:59:00           EXCLUSIVE-Bankman-Fried's FTX, parents bought Bahamas property worth $121 mln                                                               Sam Bankman-Fried's FTX, his parents and senior executives of the failed cryptocurrency exchange bought at least 19 properties worth nearly $121 million in the Bahamas over the past two years, official property records show.  urn:newsml:reuters.com:20221122:nL1N32I0A4  2022-11-22T061143Z_1_NT4_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:31:00  EXCLUSIVE-China set to fine Ant Group over $1 bln, signalling revamp nears end-sources                                                    Chinese authorities are poised to impose a fine of more than $1 billion on Jack Ma's Ant Group, said six sources with direct knowledge of the matter, setting the stage for ending the fintech company's two-year long regulatory overhaul.  urn:newsml:reuters.com:20221122:nL1N32I0MJ  2022-11-22T090637Z_1_SY6_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:21:00             Ukrainians brace for bleak winter as Russian strikes cripple power capacity                                    President Volodymyr Zelenskiy appealed to Ukrainians to conserve energy amid relentless Russian strikes that have already halved the country's power capacity, as the United Nations' health body warned of a humanitarian disaster in Ukraine this winter.  urn:newsml:reuters.com:20221121:nL1N32H2DO  2022-11-22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 12:07:00                      GLOBAL MARKETS-Shares brush off China COVID curbs, dollar retreats                                                           Global shares rose on Tuesday, recovering some of the previous day's losses, as improved investor risk appetite drove flows into equities and commodities, although concern over more COVID infections in China kept gains in check.  urn:newsml:reuters.com:20221122:nL1N32I0QM  2022-11-22T085147Z_1_LW3_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:06:00                                Beijing shuts parks, museums as China's COVID cases rise                                                 Beijing shut parks, malls and museums on Tuesday while more Chinese cities resumed mass testing for COVID-19 as authorities struggle with a spike in cases that has deepened concern about the economy and dimmed hopes for a quick reopening.  urn:newsml:reuters.com:20221122:nL1N32I03J  2022-11-22T112536Z_1_FB1_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:09:00                                                 Cryptoverse: Let's talk about DEX, baby                                                                                                                                                                  As the crypto castle crumbles, some true believers say the answer is to double down on DEX. Decentralized exchanges, that is.  urn:newsml:reuters.com:20221122:nL4N32E3CU  2022-11-22T062251Z_1_GY2_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:59:00           ANALYSIS-Kohl's snub of big sale-leaseback sets up new clash with hedge funds                                                                                                                        Kohl's Corp's reluctance to sell much of its real estate and then lease it back has set up the U.S. department store operator for a new confrontation with hedge funds.  urn:newsml:reuters.com:20221122:nL1N32B2UJ  2022-11-22T111327Z_1_JZ3_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:32:00                           Don't like Musk? Work for us! Tech firms woo ex-Twitter staff                                                Put off by Elon Musk's muscular management style? Move to us! That's the pitch being used by talent-starved technology firms trying to lure thousands of former Twitter Inc employees laid off by the social media company under its new owner.  urn:newsml:reuters.com:20221122:nL8N32H3XG  2022-11-22T104018Z_1_RH6_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 09:43:00        ANALYSIS-China has limited power, and perhaps little desire, to curb North Korea                                                                U.S. President Joe Biden last week asked his Chinese counterpart, Xi Jinping, if he could try to talk North Korea out of a new nuclear weapon test but he acknowledged that he did not know if China had the capacity to do so.  urn:newsml:reuters.com:20221122:nL4N32F06Z  2022-11-22T095230Z_1_QY5_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 09:45:00               EU gas cap seen in place for one year, countries at odds over price level                                                                                                The European Commission proposed introducing a gas price cap for one year from Jan.1, 2023, according to draft legislation seen by Reuters that has so far left the actual ceiling level blank.  urn:newsml:reuters.com:20221122:nL8N32I2CE  2022-11-22T092043Z_1_WT7_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:59:00                 FOCUS-Generic drugmakers Teva and Sandoz make major push to biosimilars  Generic drug makers Teva Pharmaceutical Industries and Sandoz say they are planning a significant ramp-up in production of biosimilars – copies of high-priced drugs used to treat illnesses such as rheumatoid arthritis and cancer – aiming to increase their share of an expanding market.  urn:newsml:reuters.com:20221122:nL6N32D0C3  2022-11-22T110845Z_1_FN2_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 05:29:00               'Boomerang CEOs' don't always work out; Disney hopes this one bucks trend                                                                                                          Can Walt Disney Co bank on another hit sequel? That appears to be the hope behind the company's surprise decision to bring back former chief executive BobIger to replace Bob Chapek.  urn:newsml:reuters.com:20221122:nL1N32H1O7  2022-11-22T111759Z_1_YH8_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:38:00                            Many school children among 252 dead in Indonesian earthquake                                                                                      Many children were killed in an earthquake in Indonesia when their schools collapsed, officials said on Tuesday, as rescuers raced to reach people trapped in rubble in a devastated town on Java island.  urn:newsml:reuters.com:20221122:nL1N32I01R  2022-11-22T093118Z_1_UY6_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 09:21:00          China's COVID resurgence delays oil demand recovery to after Q2 2023 -analysts                                                                     Analysts are cutting forecasts for China's year-end oil demand after COVID-19 cases surged to near record levels, forcing authorities to reinstate mobility curbs and delaying recovery at the world's top crude importer.  urn:newsml:reuters.com:20221122:nL1N32I0IM  2022-11-22T024400Z_1_LM3_RTRLXPP_2_LYNXPACKAGER__JPG"""
