from enum import Enum, unique


@unique
class Industry(Enum):
    ADVANCED_MEDICAL_EQUIPMENT_AND_TECHNOLOGY = "AdvancedMedicalEquipmentAndTechnology"
    ADVERTISING_AND_MARKETING = "AdvertisingAndMarketing"
    AEROSPACE_AND_DEFENSE = "AerospaceAndDefense"
    AGRICULTURAL_CHEMICALS = "AgriculturalChemicals"
    AIRLINES = "Airlines"
    AIRPORT_OPERATORS_AND_SERVICES = "AirportOperatorsAndServices"
    ALUMINUM = "Aluminum"
    APPAREL_AND_ACCESSORIES = "ApparelAndAccessories"
    APPAREL_AND_ACCESSORIES_RETAILERS = "ApparelAndAccessoriesRetailers"
    APPLIANCES_TOOLS_AND_HOUSEWARES = "AppliancesToolsAndHousewares"
    AUTO_AND_TRUCK_MANUFACTURERS = "AutoAndTruckManufacturers"
    AUTO_TRUCK_AND_MOTORCYCLE_PARTS = "AutoTruckAndMotorcycleParts"
    AUTO_VEHICLES_PARTS_AND_SERVICE_RETAILERS = "AutoVehiclesPartsAndServiceRetailers"
    BANKS = "Banks"
    BIOTECHNOLOGY_AND_MEDICAL_RESEARCH = "BiotechnologyAndMedicalResearch"
    BLOCKCHAIN_AND_CRYPTOCURRENCY = "BlockchainAndCryptocurrency"
    BREWERS = "Brewers"
    BROADCASTING = "Broadcasting"
    BUSINESS_SUPPORT_SERVICES = "BusinessSupportServices"
    BUSINESS_SUPPORT_SUPPLIES = "BusinessSupportSupplies"
    CASINOS_AND_GAMING = "CasinosAndGaming"
    CHARITY_ORGANIZATIONS = "CharityOrganizations"
    CIVIC_AND_SOCIAL_ORGANIZATIONS = "CivicAndSocialOrganizations"
    CLOSED_END_FUNDS = "ClosedEndFunds"
    COAL = "Coal"
    COMMERCIAL_PRINTING_SERVICES = "CommercialPrintingServices"
    COMMERCIAL_REI_TS = "CommercialREITs"
    COMMODITY_CHEMICALS = "CommodityChemicals"
    COMMUNICATIONS_AND_NETWORKING = "CommunicationsAndNetworking"
    COMPUTER_AND_ELECTRONICS_RETAILERS = "ComputerAndElectronicsRetailers"
    COMPUTER_HARDWARE = "ComputerHardware"
    CONSTRUCTION_AND_ENGINEERING = "ConstructionAndEngineering"
    CONSTRUCTION_MATERIALS = "ConstructionMaterials"
    CONSTRUCTION_SUPPLIES_AND_FIXTURES = "ConstructionSuppliesAndFixtures"
    CONSUMER_GOODS_CONGLOMERATES = "ConsumerGoodsConglomerates"
    CONSUMER_LENDING = "ConsumerLending"
    CONSUMER_PUBLISHING = "ConsumerPublishing"
    CORPORATE_FINANCIAL_SERVICES = "CorporateFinancialServices"
    COURIER_POSTAL_AIR_FREIGHT_AND_LANDBASED_LOGISTICS = (
        "CourierPostalAirFreightAndLandbasedLogistics"
    )
    CROWD_COLLABORATION = "CrowdCollaboration"
    DEPARTMENT_STORES = "DepartmentStores"
    DISCOUNT_STORES = "DiscountStores"
    DISTILLERS_AND_WINERIES = "DistillersAndWineries"
    DIVERSIFIED_CHEMICALS = "DiversifiedChemicals"
    DIVERSIFIED_INDUSTRIAL_GOODS_WHOLESALE = "DiversifiedIndustrialGoodsWholesale"
    DIVERSIFIED_INVESTMENT_SERVICES = "DiversifiedInvestmentServices"
    DIVERSIFIED_MINING = "DiversifiedMining"
    DIVERSIFIED_REI_TS = "DiversifiedREITs"
    DRUG_RETAILERS = "DrugRetailers"
    ELECTRIC_UTILITIES = "ElectricUtilities"
    ELECTRICAL_COMPONENTS_AND_EQUIPMENT = "ElectricalComponentsAndEquipment"
    ELECTRONIC_EQUIPMENT_AND_PARTS = "ElectronicEquipmentAndParts"
    EMPLOYMENT_SERVICES = "EmploymentServices"
    ENTERTAINMENT_PRODUCTION = "EntertainmentProduction"
    ENVIRONMENTAL_ORGANIZATIONS = "EnvironmentalOrganizations"
    ENVIRONMENTAL_SERVICES_AND_EQUIPMENT = "EnvironmentalServicesAndEquipment"
    EXCHANGE_TRADED_FUNDS = "ExchangeTradedFunds"
    FINANCIAL_AND_COMMODITY_MARKET_OPERATORS_AND_SERVICE_PROVIDERS = (
        "FinancialAndCommodityMarketOperatorsAndServiceProviders"
    )
    FINANCIAL_TECHNOLOGY = "FinancialTechnology"
    FISHING_AND_FARMING = "FishingAndFarming"
    FOOD_PROCESSING = "FoodProcessing"
    FOOD_RETAIL_AND_DISTRIBUTION = "FoodRetailAndDistribution"
    FOOTWEAR = "Footwear"
    FOREST_AND_WOOD_PRODUCTS = "ForestAndWoodProducts"
    GOLD = "Gold"
    GOVERNMENT_ADMINISTRATION_ACTIVITIES = "GovernmentAdministrationActivities"
    GOVERNMENT_AND_GOVERNMENT_FINANCE = "GovernmentAndGovernmentFinance"
    GROUND_FREIGHT_AND_LOGISTICS = "GroundFreightAndLogistics"
    HEALTHCARE_FACILITIES_AND_SERVICES = "HealthcareFacilitiesAndServices"
    HEAVY_ELECTRICAL_EQUIPMENT = "HeavyElectricalEquipment"
    HEAVY_MACHINERY_AND_VEHICLES = "HeavyMachineryAndVehicles"
    HIGHWAYS_AND_RAIL_TRACKS = "HighwaysAndRailTracks"
    HOME_FURNISHINGS = "HomeFurnishings"
    HOME_FURNISHINGS_RETAILERS = "HomeFurnishingsRetailers"
    HOME_IMPROVEMENT_PRODUCTS_AND_SERVICES_RETAILERS = (
        "HomeImprovementProductsAndServicesRetailers"
    )
    HOMEBUILDING = "Homebuilding"
    HOTELS_MOTELS_AND_CRUISE_LINES = "HotelsMotelsAndCruiseLines"
    HOUSEHOLD_ELECTRONICS = "HouseholdElectronics"
    HOUSEHOLD_PRODUCTS = "HouseholdProducts"
    IT_SERVICES_AND_CONSULTING = "ITServicesAndConsulting"
    INDEPENDENT_POWER_PRODUCERS = "IndependentPowerProducers"
    INDUSTRIAL_MACHINERY_AND_EQUIPMENT = "IndustrialMachineryAndEquipment"
    INSURANCE_FUNDS = "InsuranceFunds"
    INTEGRATED_HARDWARE_AND_SOFTWARE = "IntegratedHardwareAndSoftware"
    INTEGRATED_OIL_AND_GAS = "IntegratedOilAndGas"
    INTEGRATED_TELECOMMUNICATIONS_SERVICES = "IntegratedTelecommunicationsServices"
    INVESTMENT_BANKING_AND_BROKERAGE_SERVICES = "InvestmentBankingAndBrokerageServices"
    INVESTMENT_HOLDING_COMPANIES = "InvestmentHoldingCompanies"
    INVESTMENT_MANAGEMENT_AND_FUND_OPERATORS = "InvestmentManagementAndFundOperators"
    IRON_AND_STEEL = "IronAndSteel"
    LEGAL_AND_SAFETY_PUBLIC_SERVICES = "LegalAndSafetyPublicServices"
    LEISURE_AND_RECREATION = "LeisureAndRecreation"
    LIFE_AND_HEALTH_INSURANCE = "LifeAndHealthInsurance"
    MANAGED_HEALTHCARE = "ManagedHealthcare"
    MARINE_FREIGHT_AND_LOGISTICS = "MarineFreightAndLogistics"
    MARINE_PORT_SERVICES = "MarinePortServices"
    MEDICAL_EQUIPMENT_SUPPLIES_AND_DISTRIBUTION = (
        "MedicalEquipmentSuppliesAndDistribution"
    )
    MINING_SUPPORT_SERVICES_AND_EQUIPMENT = "MiningSupportServicesAndEquipment"
    MISCELLANEOUS_EDUCATIONAL_SERVICE_PROVIDERS = (
        "MiscellaneousEducationalServiceProviders"
    )
    MISCELLANEOUS_INFRASTRUCTURE = "MiscellaneousInfrastructure"
    MISCELLANEOUS_SPECIALTY_RETAILERS = "MiscellaneousSpecialtyRetailers"
    MULTILINE_INSURANCE_AND_BROKERS = "MultilineInsuranceAndBrokers"
    MULTILINE_UTILITIES = "MultilineUtilities"
    MUTUAL_FUNDS = "MutualFunds"
    NATIONAL_SECURITY_AND_INTERNATIONAL_AFFAIRS = (
        "NationalSecurityAndInternationalAffairs"
    )
    NATURAL_GAS_UTILITIES = "NaturalGasUtilities"
    NON_ALCOHOLIC_BEVERAGES = "NonAlcoholicBeverages"
    NON_GOLD_PRECIOUS_METALS_AND_MINERALS = "NonGoldPreciousMetalsAndMinerals"
    NON_PAPER_CONTAINERS_AND_PACKAGING = "NonPaperContainersAndPackaging"
    OFFICE_EQUIPMENT = "OfficeEquipment"
    OIL_AND_GAS_DRILLING = "OilAndGasDrilling"
    OIL_AND_GAS_EXPLORATION_AND_PRODUCTION = "OilAndGasExplorationAndProduction"
    OIL_AND_GAS_REFINING_AND_MARKETING = "OilAndGasRefiningAndMarketing"
    OIL_AND_GAS_TRANSPORTATION_SERVICES = "OilAndGasTransportationServices"
    OIL_RELATED_SERVICES_AND_EQUIPMENT = "OilRelatedServicesAndEquipment"
    ONLINE_SERVICES = "OnlineServices"
    PAPER_PACKAGING = "PaperPackaging"
    PAPER_PRODUCTS = "PaperProducts"
    PASSENGER_TRANSPORTATION_GROUND_AND_SEA = "PassengerTransportationGroundAndSea"
    PENSION_FUNDS = "PensionFunds"
    PERSONAL_PRODUCTS = "PersonalProducts"
    PERSONAL_SERVICES = "PersonalServices"
    PHARMACEUTICALS = "Pharmaceuticals"
    PHONES_AND_HANDHELD_DEVICES = "PhonesAndHandheldDevices"
    PROFESSIONAL_AND_BUSINESS_EDUCATION = "ProfessionalAndBusinessEducation"
    PROFESSIONAL_INFORMATION_SERVICES = "ProfessionalInformationServices"
    PROFESSIONAL_ORGANIZATIONS = "ProfessionalOrganizations"
    PROPERTY_AND_CASUALTY_INSURANCE = "PropertyAndCasualtyInsurance"
    REAL_ESTATE_RENTAL_DEVELOPMENT_AND_OPERATIONS = (
        "RealEstateRentalDevelopmentAndOperations"
    )
    REAL_ESTATE_SERVICES = "RealEstateServices"
    RECREATIONAL_PRODUCTS = "RecreationalProducts"
    REINSURANCE = "Reinsurance"
    RELIGIOUS_ORGANIZATIONS = "ReligiousOrganizations"
    RENEWABLE_ENERGY_EQUIPMENT_AND_SERVICES = "RenewableEnergyEquipmentAndServices"
    RENEWABLE_FUELS = "RenewableFuels"
    RESIDENTIAL_REI_TS = "ResidentialREITs"
    RESTAURANTS_AND_BARS = "RestaurantsAndBars"
    SCHOOL_COLLEGE_AND_UNIVERSITY = "SchoolCollegeAndUniversity"
    SEMICONDUCTOR_EQUIPMENT_AND_TESTING = "SemiconductorEquipmentAndTesting"
    SEMICONDUCTORS = "Semiconductors"
    SHIPBUILDING = "Shipbuilding"
    SOFTWARE = "Software"
    SPECIALIZED_REI_TS = "SpecializedREITs"
    SPECIALTY_CHEMICALS = "SpecialtyChemicals"
    SPECIALTY_MINING_AND_METALS = "SpecialtyMiningAndMetals"
    TEXTILES_AND_LEATHER_GOODS = "TextilesAndLeatherGoods"
    TIRES_AND_RUBBER_PRODUCTS = "TiresAndRubberProducts"
    TOBACCO = "Tobacco"
    TOYS_AND_CHILDREN_PRODUCTS = "ToysAndChildrenProducts"
    UK_INVESTMENT_TRUSTS = "UKInvestmentTrusts"
    URANIUM = "Uranium"
    WATER_AND_RELATED_UTILITIES = "WaterAndRelatedUtilities"
    WIRELESS_TELECOMMUNICATIONS_SERVICES = "WirelessTelecommunicationsServices"
