from enum import Enum, unique


@unique
class IndustryGroup(Enum):
    AEROSPACE_AND_DEFENSE = "AerospaceAndDefense"
    AUTOMOBILES_AND_AUTO_PARTS = "AutomobilesAndAutoParts"
    BANKING_AND_INVESTMENT_SERVICES = "BankingAndInvestmentServices"
    BANKING_SERVICES = "BankingServices"
    BEVERAGES = "Beverages"
    BIOTECHNOLOGY_AND_MEDICAL_RESEARCH = "BiotechnologyAndMedicalResearch"
    CHEMICALS = "Chemicals"
    COAL = "Coal"
    COLLECTIVE_INVESTMENTS = "CollectiveInvestments"
    COMMUNICATIONS_AND_NETWORKING = "CommunicationsAndNetworking"
    COMPUTERS_PHONES_AND_HOUSEHOLD_ELECTRONICS = (
        "ComputersPhonesAndHouseholdElectronics"
    )
    CONSTRUCTION_AND_ENGINEERING = "ConstructionAndEngineering"
    CONSTRUCTION_MATERIALS = "ConstructionMaterials"
    CONSUMER_GOODS_CONGLOMERATES = "ConsumerGoodsConglomerates"
    CONTAINERS_AND_PACKAGING = "ContainersAndPackaging"
    DIVERSIFIED_INDUSTRIAL_GOODS_WHOLESALE = "DiversifiedIndustrialGoodsWholesale"
    DIVERSIFIED_RETAIL = "DiversifiedRetail"
    ELECTRIC_UTILITIES_AND_IP_PS = "ElectricUtilitiesAndIPPs"
    ELECTRONIC_EQUIPMENT_AND_PARTS = "ElectronicEquipmentAndParts"
    FINANCIAL_TECHNOLOGY_AND_INFRASTRUCTURE = "FinancialTechnologyAndInfrastructure"
    FOOD_AND_DRUG_RETAILING = "FoodAndDrugRetailing"
    FOOD_AND_TOBACCO = "FoodAndTobacco"
    FREIGHT_AND_LOGISTICS_SERVICES = "FreightAndLogisticsServices"
    GOVERNMENT_ACTIVITY = "GovernmentActivity"
    HEALTHCARE_EQUIPMENT_AND_SUPPLIES = "HealthcareEquipmentAndSupplies"
    HEALTHCARE_PROVIDERS_AND_SERVICES = "HealthcareProvidersAndServices"
    HOMEBUILDING_AND_CONSTRUCTION_SUPPLIES = "HomebuildingAndConstructionSupplies"
    HOTELS_AND_ENTERTAINMENT_SERVICES = "HotelsAndEntertainmentServices"
    HOUSEHOLD_GOODS = "HouseholdGoods"
    INSTITUTIONS_ASSOCIATIONS_AND_ORGANIZATIONS = (
        "InstitutionsAssociationsAndOrganizations"
    )
    INSURANCE = "Insurance"
    INTEGRATED_HARDWARE_AND_SOFTWARE = "IntegratedHardwareAndSoftware"
    INVESTMENT_BANKING_AND_INVESTMENT_SERVICES = (
        "InvestmentBankingAndInvestmentServices"
    )
    INVESTMENT_HOLDING_COMPANIES = "InvestmentHoldingCompanies"
    LEISURE_PRODUCTS = "LeisureProducts"
    MACHINERY_TOOLS_HEAVY_VEHICLES_TRAINS_AND_SHIPS = (
        "MachineryToolsHeavyVehiclesTrainsAndShips"
    )
    MEDIA_AND_PUBLISHING = "MediaAndPublishing"
    METALS_AND_MINING = "MetalsAndMining"
    MISCELLANEOUS_EDUCATIONAL_SERVICE = "MiscellaneousEducationalService"
    MULTILINE_UTILITIES = "MultilineUtilities"
    NATURAL_GAS_UTILITIES = "NaturalGasUtilities"
    OFFICE_EQUIPMENT = "OfficeEquipment"
    OIL_AND_GAS = "OilAndGas"
    OIL_AND_GAS_RELATED_EQUIPMENT_AND_SERVICES = "OilAndGasRelatedEquipmentAndServices"
    PAPER_AND_FOREST_PRODUCTS = "PaperAndForestProducts"
    PASSENGER_TRANSPORTATION_SERVICES = "PassengerTransportationServices"
    PERSONAL_AND_HOUSEHOLD_PRODUCTS_AND_SERVICES = (
        "PersonalAndHouseholdProductsAndServices"
    )
    PHARMACEUTICALS = "Pharmaceuticals"
    PROFESSIONAL_AND_BUSINESS_EDUCATION = "ProfessionalAndBusinessEducation"
    PROFESSIONAL_AND_COMMERCIAL_SERVICES = "ProfessionalAndCommercialServices"
    PROVIDERS = "Providers"
    REAL_ESTATE_OPERATIONS = "RealEstateOperations"
    RENEWABLE_ENERGY = "RenewableEnergy"
    RESIDENTIAL_AND_COMMERCIAL_REI_TS = "ResidentialAndCommercialREITs"
    SCHOOL_COLLEGE_AND_UNIVERSITY = "SchoolCollegeAndUniversity"
    SEMICONDUCTORS_AND_SEMICONDUCTOR_EQUIPMENT = (
        "SemiconductorsAndSemiconductorEquipment"
    )
    SOFTWARE_AND_IT_SERVICES = "SoftwareAndITServices"
    SPECIALTY_RETAILERS = "SpecialtyRetailers"
    TELECOMMUNICATIONS_SERVICES = "TelecommunicationsServices"
    TEXTILES_AND_APPAREL = "TextilesAndApparel"
    TRANSPORT_INFRASTRUCTURE = "TransportInfrastructure"
    URANIUM = "Uranium"
    WATER_AND_RELATED_UTILITIES = "WaterAndRelatedUtilities"
