from typing import Optional


from ._curves._credit_curve_definition import CreditCurveDefinition
from ._curves._credit_curve_parameters import CreditCurveParameters


CurveDefinition = Optional[CreditCurveDefinition]
CurveParameters = Optional[CreditCurveParameters]
