data = {
    "services": [
        {
            "port": 14002,
            "location": ["ap-southeast-1a"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "ap-southeast-1-aws-1-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [2001, 3000],
        },
        {
            "port": 14002,
            "location": ["ap-southeast-1a", "ap-southeast-1b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "ap-southeast-1-aws-3-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["ap-southeast-1b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "ap-southeast-1-aws-2-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["eu-west-1a"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "eu-west-1-aws-1-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["eu-west-1a", "eu-west-1b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "eu-west-1-aws-3-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["eu-west-1b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "eu-west-1-aws-2-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["us-east-1a"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "us-east-1-aws-1-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["us-east-1a", "us-east-1b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "us-east-1-aws-3-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["us-east-1b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "us-east-1-aws-2-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["us-east-2a"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "us-east-2-aws-1-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["us-east-2a", "us-east-2b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "us-east-2-aws-3-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 14002,
            "location": ["us-east-2b"],
            "transport": "tcp",
            "provider": "aws",
            "endpoint": "us-east-2-aws-2-med.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["rwf"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["ap-southeast-1a"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "ap-southeast-1-aws-1-lrg.optimized-pricing/api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["ap-southeast-1a", "ap-southeast-1b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "ap-southeast-1-aws-3-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["ap-southeast-1b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "ap-southeast-1-aws-2-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["eu-west-1a"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "eu-west-1-aws-1-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["eu-west-1a", "eu-west-1b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "eu-west-1-aws-3-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["eu-west-1b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "eu-west-1-aws-2-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["us-east-1a"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "us-east-1-aws-1-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["us-east-1a", "us-east-1b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "us-east-1-aws-3-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["us-east-1b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "us-east-1-aws-2-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["us-east-2a"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "us-east-2-aws-1-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["us-east-2a", "us-east-2b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "us-east-2-aws-3-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
        {
            "port": 443,
            "location": ["us-east-2b"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "us-east-2-aws-2-lrg.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
            "tier": [1000, 2000],
        },
    ]
}
