SCOPE_BY_KEY = {
    ("/user-framework/mobile/news-service/v1/articles/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/articles/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/articles/{id}", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/news-service/v1/articles/{id}/translation/{language}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/news-service/v1/articles/{id}/translation/{language}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/news-service/v1/articles/{id}/translation/{language}",
        "head",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/news-service/v1/categories", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/categories", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/digest/pnd", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/digest/pnd", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/digest/pnd", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/digest/reports", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/digest/reports", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/breaking", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/breaking", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/external-url/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/external-url/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/headlines", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/headlines", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/is-read", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/is-read", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/mark-as-read", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/mark-as-read", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read/keyword", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read/keyword", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read/keyword", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-read/keyword", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent/keyword", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent/keyword", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent/keyword", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/most-recent/keyword", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/news-service/v1/news/portfolio/{portfolioId}/trending",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/news-service/v1/news/portfolio/{portfolioId}/trending",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/news-service/v1/news/portfolio/{portfolioId}/volume-analysis",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/news-service/v1/news/portfolio/{portfolioId}/volume-analysis",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/news-service/v1/news/related-stories/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/related-stories/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/saved-search/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/saved-search/{id}", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/saved-search/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/saved-search/{id}", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/suggest", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/suggest", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/top", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/top", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/top", "head"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/topic/volume-analysis", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/topic/volume-analysis", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/topic/{topic}/trending", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/topic/{topic}/trending", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/topic/{topic}/trends", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/topic/{topic}/trends", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/watchlist/most-read/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/news/watchlist/most-read/{id}", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/news-service/v1/news/watchlist/most-read/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/news-service/v1/news/watchlist/most-recent/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/news-service/v1/news/watchlist/most-recent/{id}",
        "post",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/news-service/v1/news/watchlist/most-recent/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/news-service/v1/topic/most-read", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/most-read", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/most-recent", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/most-recent", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/search", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/search", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/{id}/children", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-service/v1/topic/{id}/children", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/news/source-authority/metadata/v1/contact/list", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/list", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/save", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/save", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/update", "put"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/update", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/{contactId}/entity", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/{contactId}/entity", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/{contactId}/remove", "delete"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/contact/{contactId}/remove", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/search", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/search", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/entity", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/entity", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/product-codes", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/product-codes", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/source-codes", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/source-codes", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/vendor-contacts", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/feed/{feedCode}/vendor-contacts", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/health-check/ping", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/health-check/ping", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/mrn-rules/search", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/mrn-rules/search", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/mrn-rules/{mrnPeCode}/mrn-details", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/mrn-rules/{mrnPeCode}/mrn-details",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/mrn-rules/{mrnPeCode}/product-codes", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/mrn-rules/{mrnPeCode}/product-codes",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/mrn-rules/{mrnPeCode}/source-codes", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/mrn-rules/{mrnPeCode}/source-codes",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/namedItem-code/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/namedItem-code/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/NewsMapPE", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/NewsMapPE", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/newsMapCodes", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/newsMapCodes", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/newsMapLanguages", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/newsMapLanguages", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/newsMapResults", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/newsMapResults", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/nmrproductcodes", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/nmrproductcodes", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/nmrsourcecodes", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/newsmap/nmrsourcecodes", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/Feed/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/Feed/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/MRN/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/MRN/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/permission-entity/{peCode}/entity", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/permission-entity/{peCode}/entity",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/permission-entity/{peCode}/product-definition-pages",
        "get",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/permission-entity/{peCode}/product-definition-pages",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/permission-entity/{peCode}/product-line-items",
        "get",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/permission-entity/{peCode}/product-line-items",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/product-code/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/product-code/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/product-code/search", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/product-code/search", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/product-code/{productCode}/entity", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/product-code/{productCode}/entity",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/product-code/{productCode}/feeds", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/product-code/{productCode}/feeds",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/product-code/{productCode}/permission-entities",
        "get",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/product-code/{productCode}/permission-entities",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/product-code/{productCode}/source-codes",
        "get",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/product-code/{productCode}/source-codes",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/ref/list", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/ref/list", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/search", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/search", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/{sourceCode}/entity", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/{sourceCode}/entity", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/{sourceCode}/feeds", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/{sourceCode}/feeds", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/source-code/{sourceCode}/product-codes",
        "get",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/source-code/{sourceCode}/product-codes",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/source-code/{sourceCode}/pubs", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-code/{sourceCode}/pubs", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-product-data/export", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-product-data/export", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-product-data/search", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/source-product-data/search", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/subject-code/ids", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/subject-code/ids", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/exportPermissionUserList", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/exportPermissionUserList", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/list", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/list", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/permissions", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/permissions", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/roles", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/roles", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/save", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/save", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/search", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/search", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/update", "put"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/update", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/usernames", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/usernames", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/userroles", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/userroles", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/{userId}/details", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/{userId}/details", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/user/{userId}/rolePermissionForUser", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/user/{userId}/rolePermissionForUser",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/user/{userid}/{modifiedby}/remove",
        "delete",
    ): [{"trapi.metadata.nsa.read"}],
    (
        "/news/source-authority/metadata/v1/user/{userid}/{modifiedby}/remove",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/news/source-authority/metadata/v1/vendor/assign", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/assign", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/list", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/list", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/multi-assign", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/multi-assign", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/save", "post"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/save", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/update", "put"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/update", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/{vendorId}/feeds", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/{vendorId}/feeds", "options"): [
        {"trapi.metadata.nsa.read"}
    ],
    ("/news/source-authority/metadata/v1/vendor/{vendorId}/vendor-contacts", "get"): [
        {"trapi.metadata.nsa.read"}
    ],
    (
        "/news/source-authority/metadata/v1/vendor/{vendorId}/vendor-contacts",
        "options",
    ): [{"trapi.metadata.nsa.read"}],
    ("/data/pricing/vehicle-management/v1/fieldlist", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/fieldlist", "post"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/fieldlist", "delete"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/fieldlist/{id}", "put"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/fieldlist/{id}", "delete"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/fieldlist_id", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/fieldlist_name", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service", "post"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service", "delete"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service/{id}", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service/{id}", "put"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service/{id}", "delete"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service_id", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/service_name", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehicle_id", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehicle_ids", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehiclegroup", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehiclegroup", "post"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehiclegroup/{id}", "delete"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehiclegroup_id", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehiclegroup_ids", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehiclegroup_name", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehicles", "post"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehicles/{id}", "get"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehicles/{id}", "put"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/pricing/vehicle-management/v1/vehicles/{id}", "delete"): [
        {"trapi.data.vehicle-management"}
    ],
    ("/data/filings/v1/retrieval/search/{identifier}/{value}", "get"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/filings/v1/retrieval/search/{identifier}/{value}", "options"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/filings/v1/retrieval/{filename}", "get"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/filings/v1/retrieval/{filename}", "options"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/environmental-social-governance/v2/universe", "get"): [
        {"trapi.data.esg.universe.read", "trapi.data.esg.read"},
        {"trapi.demo-access", "trapi.data.esg.read"},
    ],
    ("/data/environmental-social-governance/v2/universe", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/basic", "get"): [
        {"trapi.data.esg.views-basic.read", "trapi.data.esg.read"},
        {"trapi.demo-access", "trapi.data.esg.read"},
    ],
    ("/data/environmental-social-governance/v2/views/basic", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/dir-scores", "get"): [
        {"trapi.data.esg.views-scores-standard.read", "trapi.data.esg.read"},
        {"trapi.data.esg.views-scores-full.read", "trapi.data.esg.read"},
        {"trapi.data.esg.views-measures-standard.read", "trapi.data.esg.read"},
        {"trapi.data.esg.views-measures-full.read", "trapi.data.esg.read"},
    ],
    ("/data/environmental-social-governance/v2/views/dir-scores", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/measures-full", "get"): [
        {"trapi.data.esg.views-measures-full.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/measures-full", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/measures-partialupdates", "get"): [
        {"trapi.data.esg.views-measures-full.read", "trapi.data.esg.read"},
        {"trapi.data.esg.views-measures-standard.read", "trapi.data.esg.read"},
    ],
    (
        "/data/environmental-social-governance/v2/views/measures-partialupdates",
        "options",
    ): [{"trapi.data.esg.read"}],
    ("/data/environmental-social-governance/v2/views/measures-standard", "get"): [
        {"trapi.data.esg.views-measures-standard.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/measures-standard", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/scores-full", "get"): [
        {"trapi.data.esg.views-scores-full.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/scores-full", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/scores-standard", "get"): [
        {"trapi.data.esg.views-scores-standard.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v2/views/scores-standard", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/change-notifications/v1/allproducts", "get"): [
        {"rdpapi.data.notifi.reader"}
    ],
    ("/data/change-notifications/v1/allproducts", "options"): [],
    ("/data/change-notifications/v1/allsuppliers", "get"): [
        {"rdpapi.data.notifi.reader"}
    ],
    ("/data/change-notifications/v1/allsuppliers", "options"): [],
    ("/data/change-notifications/v1/content-services", "get"): [
        {"rdpapi.data.notifi.reader"}
    ],
    ("/data/change-notifications/v1/content-services", "options"): [],
    ("/data/change-notifications/v1/content-services-all", "get"): [
        {"rdpapi.data.notifi.reader"}
    ],
    ("/data/change-notifications/v1/content-services-all", "options"): [],
    ("/data/change-notifications/v1/lookups", "get"): [{"rdpapi.data.notifi.reader"}],
    ("/data/change-notifications/v1/lookups", "options"): [],
    ("/data/change-notifications/v1/lookups/{lookup}", "get"): [
        {"rdpapi.data.notifi.reader"}
    ],
    ("/data/change-notifications/v1/lookups/{lookup}", "options"): [],
    ("/data/change-notifications/v1/products", "get"): [{"rdpapi.data.notifi.reader"}],
    ("/data/change-notifications/v1/products", "options"): [],
    ("/data/change-notifications/v1/published-notifications", "get"): [
        {"rdpapi.data.notifi.reader"}
    ],
    ("/data/change-notifications/v1/published-notifications", "options"): [],
    (
        "/data/change-notifications/v1/published-notifications/{notification}/attachments/{attachment}/{version}",
        "get",
    ): [{"rdpapi.data.notifi.reader"}],
    (
        "/data/change-notifications/v1/published-notifications/{notification}/attachments/{attachment}/{version}",
        "options",
    ): [],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}",
        "get",
    ): [{"rdpapi.data.notifi.reader"}],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}",
        "options",
    ): [],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions",
        "get",
    ): [{"rdpapi.data.notifi.reader"}],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions",
        "options",
    ): [],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions/{revision}",
        "get",
    ): [{"rdpapi.data.notifi.reader"}],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions/{revision}",
        "options",
    ): [],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions/{revision}/diff/{changed-revision}",
        "get",
    ): [{"rdpapi.data.notifi.reader"}],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions/{revision}/diff/{changed-revision}",
        "options",
    ): [],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions/{revision}/ric-contents-file",
        "get",
    ): [{"rdpapi.data.notifi.reader"}],
    (
        "/data/change-notifications/v1/published-notifications/{published-notification}/revisions/{revision}/ric-contents-file",
        "options",
    ): [],
    ("/data/change-notifications/v1/rctts", "get"): [{"rdpapi.data.notifi.reader"}],
    ("/data/change-notifications/v1/rctts", "options"): [],
    ("/data/change-notifications/v1/suppliers", "get"): [{"rdpapi.data.notifi.reader"}],
    ("/data/change-notifications/v1/suppliers", "options"): [],
    ("/discovery/search/v1", "post"): [{"trapi.search.read"}],
    ("/discovery/search/v1", "options"): [{"trapi.search.read"}],
    ("/discovery/search/v1/explore", "post"): [
        {"trapi.search.read", "trapi.search.explore.read"},
        {"trapi.search.read", "trapi.demo-access"},
    ],
    ("/discovery/search/v1/explore", "options"): [{"trapi.search.read"}],
    ("/discovery/search/v1/health", "get"): [
        {"trapi.search.read", "trapi.search.explore.read"},
        {"trapi.search.read", "trapi.demo-access"},
    ],
    ("/discovery/search/v1/health", "options"): [{"trapi.search.read"}],
    ("/discovery/search/v1/lookup", "post"): [
        {"trapi.search.read", "trapi.search.lookup.read"}
    ],
    ("/discovery/search/v1/lookup", "options"): [
        {"trapi.search.read", "trapi.search.lookup.read"}
    ],
    ("/discovery/search/v1/metadata/views/{view}", "get"): [
        {"trapi.search.read", "trapi.search.metadata.read"},
        {"trapi.search.read", "trapi.demo-access"},
    ],
    ("/discovery/search/v1/metadata/views/{view}", "options"): [
        {"trapi.search.read", "trapi.search.metadata.read"}
    ],
    ("/docs/v1/config", "get"): [],
    ("/docs/v1/config", "options"): [],
    ("/docs/v1/definitions", "get"): [],
    ("/docs/v1/definitions", "options"): [],
    ("/docs/v1/definitions/{id}", "get"): [],
    ("/docs/v1/definitions/{id}", "options"): [],
    ("/docs/v1/definitions/{id}/demoviews", "get"): [],
    ("/docs/v1/definitions/{id}/demoviews", "options"): [],
    ("/docs/v1/definitions/{id}/examples", "get"): [],
    ("/docs/v1/definitions/{id}/examples", "options"): [],
    ("/docs/v1/definitions/{id}/readme", "get"): [],
    ("/docs/v1/definitions/{id}/readme", "options"): [],
    ("/docs/v1/definitions/{id}/swagger", "get"): [],
    ("/docs/v1/definitions/{id}/swagger", "options"): [],
    ("/docs/v1/feedback", "options"): [],
    ("/docs/v1/feedback", "post"): [],
    ("/data/ownership/views/ultimate-beneficial-ownership-availability/v2", "get"): [
        {"trapi.data.ownership.ubo.read"}
    ],
    (
        "/data/ownership/views/ultimate-beneficial-ownership-availability/v2",
        "options",
    ): [{"trapi.data.ownership.ubo.read"}],
    ("/qa/beta1/body/mime/types", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/body/mime/types", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/body/mime/types", "patch"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/body/parameters/additional/properties", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/body/parameters/additional/properties", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/body/parameters/additional/properties", "patch"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/body/parameters/array/objects", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/body/parameters/mixed/{path_value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/body/parameters/primitive", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/body/parameters/types", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/body/parameters/types", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/compression", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/cors", "options"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/data/types/format/{value}", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/data/types/format/{value}", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/data/types/format/{value}", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/data/types/multipart", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/data/types/multipart/array", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/formdata/parameters/types", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/getStatus/{status_code}", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/header/parameters/types", "get"): [
        {"trapi.deployment.crud"},
        {"trapi.deployment.crud", "trapi.graphql.subscriber.access"},
    ],
    ("/qa/beta1/lambda", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/lambda/proxy/integration/base64EncodeBody/{value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/lambda/proxy/integration/{value}", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/passthrough/behaviors/never", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/passthrough/behaviors/never", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/passthrough/behaviors/never", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/passthrough/behaviors/when/no/match", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/passthrough/behaviors/when/no/match", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/passthrough/behaviors/when/no/match", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/passthrough/behaviors/when/no/template", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/passthrough/behaviors/when/no/template", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/passthrough/behaviors/when/no/template", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/path/parameters/backendurlwithplainpath/{value}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/path/parameters/backendurlwithplainpath/{value}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/path/parameters/common/{value}", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/path/parameters/common/{value}", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/path/parameters/multipleof/{value}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/path/parameters/multipleof/{value}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/path/parameters/several/first/{value1}/second/{value2}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/path/parameters/types/{value}", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/path/parameters/types/{value}", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/path/parameters/types/{value}", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/path/parameters/types/{value}", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/query/parameters/multivalue", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/query/parameters/multivalue", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/query/parameters/types", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/query/parameters/types", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit/bucket/operation", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit/bucket/user", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit/dailyRequest", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit/dailyVolume", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit/tiered", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/rateLimit/tiered/real", "post"): [
        {"trapi.deployment.crud", "trapi.demo-access"},
        {"trapi.deployment.crud", "trapi.admin.accounts"},
    ],
    ("/qa/beta1/rateLimit/volumeBased", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/redirect", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/route", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/route/body", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/route/client/certificate", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/route/query/and/header", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/route/query/and/header", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/route/{value}", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/scopes/and", "get"): [
        {
            "trapi.admin.accounts.groups",
            "trapi.deployment.crud",
            "trapi.admin.accounts.licenses",
        }
    ],
    ("/qa/beta1/scopes/or", "get"): [
        {"trapi.admin.accounts.groups", "trapi.deployment.crud"},
        {"trapi.deployment.crud", "trapi.admin.accounts.licenses"},
    ],
    ("/qa/beta1/templates/request", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/templates/request", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/templates/request/base64EncodeBody", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/templates/request/params/{path_param}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/templates/request/utils", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/templates/request/utils", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/templates/response/{path_param}", "get"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/templates/response/{path_param}", "put"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/templates/response/{path_param}", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/test/ping", "get"): [{"trapi.deployment.crud", "trapi.data.api.test"}],
    ("/qa/beta1/test/userdelegation", "get"): [
        {"trapi.deployment.crud"},
        {"trapi.deployment.crud", "trapi.data.pricing.read"},
    ],
    ("/qa/beta1/xtr/integration/cache/body", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/xtr/integration/cache/header", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/xtr/integration/cache/path/{id}/{cache_value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/beta1/xtr/integration/cache/query", "post"): [{"trapi.deployment.crud"}],
    ("/qa/beta1/xtr/integration/request/parameters/{value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/auth/oauth2/v2/token", "post"): [],
    ("/data/formula-analytics-snapshot/v1/formulas/{formulaId}", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/formula-analytics-snapshot/v1/formulas/{formulaId}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/historical-pricing/v1/views/events", "post"): [
        {"trapi.data.historical-pricing.events.read"}
    ],
    ("/data/historical-pricing/v1/views/events", "options"): [],
    ("/data/historical-pricing/v1/views/events/{universe}", "get"): [
        {"trapi.data.historical-pricing.events.read"}
    ],
    ("/data/historical-pricing/v1/views/events/{universe}", "options"): [],
    ("/data/historical-pricing/v1/views/interday-summaries", "post"): [
        {"trapi.data.historical-pricing.summaries.read"}
    ],
    ("/data/historical-pricing/v1/views/interday-summaries", "options"): [],
    ("/data/historical-pricing/v1/views/interday-summaries/{universe}", "get"): [
        {"trapi.data.historical-pricing.summaries.read"}
    ],
    ("/data/historical-pricing/v1/views/interday-summaries/{universe}", "options"): [],
    ("/data/historical-pricing/v1/views/intraday-summaries/{universe}", "get"): [
        {"trapi.data.historical-pricing.summaries.read"}
    ],
    ("/data/historical-pricing/v1/views/intraday-summaries/{universe}", "options"): [],
    ("/data/historical-pricing/v1/views/single-event/{universe}", "get"): [
        {"trapi.data.historical-pricing.events.read"}
    ],
    ("/data/historical-pricing/v1/views/single-event/{universe}", "options"): [],
    ("/user-framework/bold/beta1/knowledge-graph-score", "post"): [
        {"trapi.data.api.test"}
    ],
    ("/user-framework/bold/beta1/knowledge-graph-score", "options"): [
        {"trapi.data.api.test"}
    ],
    ("/user-framework/bold/beta1/knowledge-graph-score/{token}/result", "get"): [
        {"trapi.data.api.test"}
    ],
    ("/user-framework/bold/beta1/knowledge-graph-score/{token}/status", "get"): [
        {"trapi.data.api.test"}
    ],
    ("/data/funds/v1/assets", "get"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/assets", "post"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/assets", "options"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/assets/ref/datapoints", "get"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/assets/ref/datapoints", "options"): [
        {"trapi.data.funds.assets.read"}
    ],
    ("/data/funds/v1/assets/ref/lookups", "get"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/assets/ref/lookups", "options"): [
        {"trapi.data.funds.assets.read"}
    ],
    ("/data/funds/v1/assets/{id}", "get"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/assets/{id}", "options"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/healthcheck", "get"): [{"trapi.data.funds.assets.read"}],
    ("/data/funds/v1/healthcheck", "options"): [{"trapi.data.funds.assets.read"}],
    ("/data/pricing/snapshots-resolve/v1/chains/{chainRIC}", "get"): [
        {"trapi.data.pricing.read"}
    ],
    ("/data/pricing/snapshots-resolve/v1/chains/{chainRIC}", "options"): [
        {"trapi.data.pricing.read"}
    ],
    ("/data/pricing/snapshots-resolve/v1/chains/{chainRIC}", "post"): [
        {"trapi.data.pricing.read"}
    ],
    ("/data/pricing/snapshots-resolve/v1/lists", "get"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/snapshots-resolve/v1/lists", "options"): [
        {"trapi.data.pricing.read"}
    ],
    ("/data/pricing/snapshots-resolve/v1/lists", "post"): [{"trapi.data.pricing.read"}],
    ("/user-data/lists/beta2/lists", "get"): [],
    ("/user-data/lists/beta2/lists", "post"): [],
    ("/user-data/lists/beta2/lists", "options"): [],
    ("/user-data/lists/beta2/lists/{listID}", "get"): [],
    ("/user-data/lists/beta2/lists/{listID}", "put"): [],
    ("/user-data/lists/beta2/lists/{listID}", "delete"): [],
    ("/user-data/lists/beta2/lists/{listID}", "options"): [],
    ("/user-data/lists/beta2/lists/{listID}/items", "post"): [],
    ("/user-data/lists/beta2/lists/{listID}/items", "options"): [],
    ("/user-data/lists/beta2/lists/{listID}/items/{itemID}", "put"): [],
    ("/user-data/lists/beta2/lists/{listID}/items/{itemID}", "delete"): [],
    ("/user-data/lists/beta2/lists/{listID}/items/{itemID}", "options"): [],
    ("/user-data/lists/beta2/metamodel", "get"): [],
    ("/user-data/lists/beta2/metamodel", "options"): [],
    ("/streaming/pricing/v1", "get"): [{"trapi.streaming.pricing.read"}],
    ("/streaming/pricing/v1", "options"): [{"trapi.streaming.pricing.read"}],
    ("/data/environmental-social-governance/beta1/universe", "get"): [
        {"trapi.data.esg.universe.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/universe", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/basic", "get"): [
        {"trapi.data.esg.views-basic.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/basic", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/measures-full", "get"): [
        {"trapi.data.esg.views-measures-full.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/measures-full", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/measures-standard", "get"): [
        {"trapi.data.esg.views-measures-standard.read", "trapi.data.esg.read"}
    ],
    (
        "/data/environmental-social-governance/beta1/views/measures-standard",
        "options",
    ): [{"trapi.data.esg.read"}],
    ("/data/environmental-social-governance/beta1/views/scores-full", "get"): [
        {"trapi.data.esg.views-scores-full.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/scores-full", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/scores-standard", "get"): [
        {"trapi.data.esg.views-scores-standard.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/beta1/views/scores-standard", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/health", "get"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/health", "options"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    (
        "/data/analytics/average-volume-analytics/v2/metadata/averagingIntervals",
        "get",
    ): [{"trapi.data.average-volume-analytics.ava_read"}],
    (
        "/data/analytics/average-volume-analytics/v2/metadata/averagingIntervals",
        "options",
    ): [{"trapi.data.average-volume-analytics.ava_read"}],
    ("/data/analytics/average-volume-analytics/v2/metadata/calculationTypes", "get"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    (
        "/data/analytics/average-volume-analytics/v2/metadata/calculationTypes",
        "options",
    ): [{"trapi.data.average-volume-analytics.ava_read"}],
    ("/data/analytics/average-volume-analytics/v2/metadata/responseCodes", "get"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/metadata/responseCodes", "options"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    (
        "/data/analytics/average-volume-analytics/v2/metadata/tradeClassifications",
        "get",
    ): [{"trapi.data.average-volume-analytics.ava_read"}],
    (
        "/data/analytics/average-volume-analytics/v2/metadata/tradeClassifications",
        "options",
    ): [{"trapi.data.average-volume-analytics.ava_read"}],
    ("/data/analytics/average-volume-analytics/v2/metadata/venues", "get"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/metadata/venues", "options"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/request", "post"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/request", "options"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/request/{ID}", "get"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/request/{ID}", "delete"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/request/{id}", "put"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    ("/data/analytics/average-volume-analytics/v2/request/{id}", "options"): [
        {"trapi.data.average-volume-analytics.ava_read"}
    ],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/async-operation/{operationId}",
        "get",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/async-operation/{operationId}",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/async-resource/{operationId}",
        "get",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/async-resource/{operationId}",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics-curves-and-surfaces/v1/async-surfaces", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-curves-and-surfaces/v1/async-surfaces", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/bond-curves/curves",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/bond-curves/curves",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/curves",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/curves",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/create",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/create",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/delete",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/delete",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/get",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/get",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/search",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/search",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/update",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/commodities-curves/definitions/update",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/curves",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/curves",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/create",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/create",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/delete",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/delete",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/get",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/get",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/search",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/search",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/update",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/definitions/update",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/triangulate-definitions/search",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/cross-currency-curves/triangulate-definitions/search",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/forward-curves",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/forward-curves",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/create",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/create",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/delete",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/delete",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/get",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/get",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/triangulate",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/triangulate",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/update",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/interest-rate-curves/definitions/update",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/taxonomy/fields-mapping/get",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/taxonomy/fields-mapping/get",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/zc-curve-definitions",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/zc-curve-definitions",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics-curves-and-surfaces/v1/curves/zc-curves", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics-curves-and-surfaces/v1/curves/zc-curves",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics-curves-and-surfaces/v1/surfaces", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-curves-and-surfaces/v1/surfaces", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/file-store/beta3/buckets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/buckets", "post"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/buckets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/buckets/{name}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/buckets/{name}", "put"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/buckets/{name}", "delete"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/buckets/{name}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/bulk-publish", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/bulk-publish", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/claim-check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/claim-check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}/status", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}/status", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}/status/{status}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}/status/{status}", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/file-sets/{id}/status/{status}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files/{id}/stream", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/files/{id}/stream", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/info/health", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/info/health", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/info/s3check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/info/s3check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/info/version", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/info/version", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages", "post"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages/{id}", "put"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/packages/{id}", "head"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.read",
            "trapi.cfs.publisher.write",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.setup.write",
            "trapi.cfs.publisher.read",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.claimcheck.read",
            "trapi.cfs.publisher.read",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.read",
            "trapi.cfs.claimcheck.write",
        },
    ],
    ("/file-store/beta3/permission/packages", "get"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta3/permission/packages", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/verification/v1/business", "post"): [{"trapi.data.verify.idv.business.read"}],
    ("/verification/v1/business", "options"): [],
    ("/verification/v1/businessverification/countrycodes", "get"): [
        {"trapi.data.verify.idv.business.read"}
    ],
    ("/verification/v1/businessverification/countrycodes", "options"): [],
    ("/verification/v1/businessverification/datafields", "get"): [
        {"trapi.data.verify.idv.business.read"}
    ],
    ("/verification/v1/businessverification/datafields", "options"): [],
    ("/verification/v1/businessverification/recommendedfields", "get"): [
        {"trapi.data.verify.idv.business.read"}
    ],
    ("/verification/v1/businessverification/recommendedfields", "options"): [],
    ("/verification/v1/document", "post"): [{"trapi.data.verify.document.qualid.read"}],
    ("/verification/v1/document", "options"): [],
    ("/verification/v1/document/{id}", "delete"): [
        {"trapi.data.verify.document.qualid.read"}
    ],
    ("/verification/v1/document/{id}", "options"): [],
    ("/verification/v1/documentlibrary", "get"): [
        {"trapi.data.verify.document.qualid.read"}
    ],
    ("/verification/v1/documentlibrary", "options"): [],
    ("/verification/v1/documentlight", "post"): [
        {"trapi.data.verify.document.qualid.read"}
    ],
    ("/verification/v1/documentlight", "options"): [],
    ("/verification/v1/healthcheck", "get"): [],
    ("/verification/v1/healthcheck", "options"): [],
    ("/verification/v1/identity", "post"): [{"trapi.data.verify.identity.qualid.read"}],
    ("/verification/v1/identity", "options"): [],
    ("/verification/v1/identityverification/countrycodes", "get"): [
        {"trapi.data.verify.identity.qualid.read"}
    ],
    ("/verification/v1/identityverification/countrycodes", "options"): [],
    ("/verification/v1/identityverification/datafields", "get"): [
        {"trapi.data.verify.identity.qualid.read"}
    ],
    ("/verification/v1/identityverification/datafields", "options"): [],
    ("/verification/v1/identityverification/recommendedfields", "get"): [
        {"trapi.data.verify.identity.qualid.read"}
    ],
    ("/verification/v1/identityverification/recommendedfields", "options"): [],
    ("/verification/v1/liveness", "post"): [{"trapi.data.verify.liveness.qualid.read"}],
    ("/verification/v1/liveness", "options"): [],
    ("/verification/v1/liveness/passive", "post"): [
        {"trapi.data.verify.liveness.qualid.read"}
    ],
    ("/verification/v1/liveness/passive", "options"): [],
    ("/verification/v1/liveness/token", "post"): [
        {"trapi.data.verify.liveness.qualid.read"}
    ],
    ("/verification/v1/liveness/token", "options"): [],
    ("/verification/v1/screening", "post"): [
        {"trapi.data.verify.screening.qualid.read"}
    ],
    ("/verification/v1/screening", "options"): [],
    ("/verification/v1/transactions", "get"): [{"trapi.data.transactions.qualid.read"}],
    ("/verification/v1/transactions", "options"): [],
    ("/verification/v1/transactions/{transactionId}", "get"): [],
    ("/verification/v1/transactions/{transactionId}", "options"): [],
    ("/verification/v1/wcprofile", "post"): [
        {"trapi.data.verify.screening.qualid.read"}
    ],
    ("/verification/v1/wcprofile", "options"): [],
    ("/data/analytics/benchmark/beta1/metadata/calculation-types", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/calculation-types", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/filter-types", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/filter-types", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/related-rics", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/related-rics", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/response-codes", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/response-codes", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/trade-classifications", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/trade-classifications", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/venues", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/metadata/venues", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/request", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/beta1/request", "post"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/categories", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/categories", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/categories/{id}/videos", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/categories/{id}/videos", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/daily", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/daily", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/search", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/newscasts-service/v1/search", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/admin/v1/api-deploy/cutover", "post"): [{"trapi.deployment.crud"}],
    ("/admin/v1/api-deploy/stage", "post"): [{"trapi.deployment.crud"}],
    ("/admin/v1/api-deploy/validate", "post"): [{"trapi.deployment.crud"}],
    ("/streaming/pricing/beta1", "get"): [{"trapi.streaming.pricing.read"}],
    ("/streaming/pricing/beta1", "options"): [{"trapi.streaming.pricing.read"}],
    ("/accounts/beta1/usage/beacon", "get"): [{"trapi.data.analytics.eikonusage"}],
    ("/accounts/beta1/usage/beacon", "options"): [{"trapi.data.analytics.eikonusage"}],
    ("/accounts/beta1/usage/export", "post"): [{"trapi.data.analytics.eikonusage"}],
    ("/accounts/beta1/usage/export", "options"): [{"trapi.data.analytics.eikonusage"}],
    ("/accounts/beta1/usage/reports", "post"): [{"trapi.data.analytics.eikonusage"}],
    ("/accounts/beta1/usage/reports", "options"): [{"trapi.data.analytics.eikonusage"}],
    ("/user-framework/workspace/v1", "post"): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/workspace/v1", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/ap", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/ap", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/calendars/{calendarId}/secret-addresses", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/calendars/{calendarId}/secret-addresses", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/workspace/v1/calendars/{calendarId}/secret-addresses",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/v1/calendars/{calendarId}/secret-addresses/{secret}",
        "delete",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/v1/calendars/{calendarId}/secret-addresses/{secret}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/v1/calendars/{calendarId}/secret-addresses/{secret}",
        "patch",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/workspace/v1/calendars/{secret}/calendar.ics", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/calendars/{secret}/calendar.ics", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/config", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/config", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/eu", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/eu", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}/versions", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}/versions", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}/versions/{version}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}/versions/{version}", "put"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}/versions/{version}", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/schemas/{type}/versions/{version}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/us", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/v1/us", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/auth/oauth2/beta1/authorize", "get"): [],
    ("/auth/oauth2/beta1/revoke", "post"): [],
    ("/auth/oauth2/beta1/token", "post"): [],
    ("/qa/openapi3/gateway/beta1/body/mime/types", "patch"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/mime/types", "post"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/body/mime/types", "put"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/body/parameters/additional/properties", "patch"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/additional/properties", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/additional/properties", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/array/objects", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/mixed/{path_value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/primitive", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/types", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/parameters/types", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/anyValue", "patch"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/anyValue", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/anyValue", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/complex", "patch"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/complex", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/complex", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/complex/not", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/nested/object", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/nested/object", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/body/schemas/not/nullable", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/compression", "post"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/cors", "options"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/data/types/file", "post"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/data/types/format/{value}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/data/types/format/{value}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/data/types/format/{value}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/data/types/multipart", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/data/types/multipart/array", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/formdata/parameters/types", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/getStatus/{status_code}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/header/parameters/types", "get"): [
        {"trapi.deployment.crud"},
        {"trapi.deployment.crud", "trapi.graphql.subscriber.access"},
    ],
    ("/qa/openapi3/gateway/beta1/lambda", "post"): [{"trapi.deployment.crud"}],
    (
        "/qa/openapi3/gateway/beta1/lambda/proxy/integration/base64EncodeBody/{value}",
        "post",
    ): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/lambda/proxy/integration/{value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/never", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/never", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/never", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/when/no/match", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/when/no/match", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/when/no/match", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/when/no/template", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/when/no/template", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/passthrough/behaviors/when/no/template", "put"): [
        {"trapi.deployment.crud"}
    ],
    (
        "/qa/openapi3/gateway/beta1/path/parameters/backendurlwithplainpath/{value}",
        "delete",
    ): [{"trapi.deployment.crud"}],
    (
        "/qa/openapi3/gateway/beta1/path/parameters/backendurlwithplainpath/{value}",
        "get",
    ): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/path/parameters/common/{value}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/path/parameters/common/{value}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/path/parameters/multipleof/{value}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/path/parameters/multipleof/{value}", "get"): [
        {"trapi.deployment.crud"}
    ],
    (
        "/qa/openapi3/gateway/beta1/path/parameters/several/first/{value1}/second/{value2}",
        "get",
    ): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/path/parameters/types/{value}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/path/parameters/types/{value}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/path/parameters/types/{value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/query/parameters/multivalue", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/query/parameters/multivalue", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/query/parameters/types", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/query/parameters/types", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit", "get"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/rateLimit/bucket/operation", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit/bucket/user", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit/dailyRequest", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit/dailyVolume", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit/tiered", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit/tiered/real", "post"): [
        {"trapi.deployment.crud", "trapi.demo-access"},
        {"trapi.deployment.crud", "trapi.admin.accounts"},
    ],
    ("/qa/openapi3/gateway/beta1/rateLimit/volumeBased", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/redirect", "post"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/route", "get"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/route/body", "post"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/route/client/certificate", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/route/query/and/header", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/route/query/and/header", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/route/{value}", "delete"): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/scopes/and", "get"): [
        {
            "trapi.admin.accounts.groups",
            "trapi.deployment.crud",
            "trapi.admin.accounts.licenses",
        }
    ],
    ("/qa/openapi3/gateway/beta1/scopes/or", "get"): [
        {"trapi.admin.accounts.groups", "trapi.deployment.crud"},
        {"trapi.deployment.crud", "trapi.admin.accounts.licenses"},
    ],
    ("/qa/openapi3/gateway/beta1/serialization/label/{id}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/serialization/label/{id}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/serialization/matrix/{id}", "delete"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/serialization/matrix/{id}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/serialization/matrix/{id}", "head"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/request", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/request", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/request/base64EncodeBody", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/request/params/{path_param}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/request/utils", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/request/utils", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/response/{path_param}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/response/{path_param}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/templates/response/{path_param}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/test/userdelegation", "get"): [
        {"trapi.deployment.crud"},
        {"trapi.deployment.crud", "trapi.data.pricing.read"},
    ],
    ("/qa/openapi3/gateway/beta1/xtr/integration/cache/body", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/xtr/integration/cache/header", "post"): [
        {"trapi.deployment.crud"}
    ],
    (
        "/qa/openapi3/gateway/beta1/xtr/integration/cache/path/{id}/{cache_value}",
        "post",
    ): [{"trapi.deployment.crud"}],
    ("/qa/openapi3/gateway/beta1/xtr/integration/cache/query", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/qa/openapi3/gateway/beta1/xtr/integration/request/parameters/{value}", "post"): [
        {"trapi.deployment.crud"}
    ],
    ("/license-management/v2/api/healthcheck", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/healthcheck", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/healthcheck-query", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/healthcheck-query", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/healthcheck-welcome-emails", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/healthcheck-welcome-emails", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/license-entitlements", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/license-entitlements", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/license-entitlements", "delete"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/license-entitlements", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/license-entitlements-reassignments", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/license-entitlements-reassignments", "options"): [
        {"trapi.platform.license-management"}
    ],
    (
        "/license-management/v2/api/license-entitlements-reassignments/{licenseReassignmentId}",
        "get",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/license-entitlements-reassignments/{licenseReassignmentId}",
        "options",
    ): [{"trapi.platform.license-management"}],
    ("/license-management/v2/api/license-entitlements/{licenseEntitlementId}", "get"): [
        {"trapi.platform.license-management"}
    ],
    (
        "/license-management/v2/api/license-entitlements/{licenseEntitlementId}",
        "options",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/license-entitlements/{licenseEntitlementId}/subscription",
        "get",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/license-entitlements/{licenseEntitlementId}/subscription",
        "options",
    ): [{"trapi.platform.license-management"}],
    ("/license-management/v2/api/licenses", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses/{licenseId}", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses/{licenseId}", "put"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses/{licenseId}", "delete"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses/{licenseId}", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses/{licenseId}/subscriptions", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/licenses/{licenseId}/subscriptions", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/subscription/license-entitlements", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/subscription/license-entitlements", "options"): [
        {"trapi.platform.license-management"}
    ],
    (
        "/license-management/v2/api/subscription/license-entitlements-reassignments",
        "post",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/subscription/license-entitlements-reassignments",
        "options",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/subscription/license-entitlements-reassignments/{subscriptionReassignmentId}",
        "get",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/subscription/license-entitlements-reassignments/{subscriptionReassignmentId}",
        "options",
    ): [{"trapi.platform.license-management"}],
    ("/license-management/v2/api/subscriptions/searches", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/subscriptions/searches", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/welcome-emails", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/welcome-emails", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/welcome-emails", "options"): [
        {"trapi.platform.license-management"}
    ],
    (
        "/license-management/v2/api/welcome-emails/requirement-for-licenses/{licenseId}",
        "get",
    ): [{"trapi.platform.license-management"}],
    (
        "/license-management/v2/api/welcome-emails/requirement-for-licenses/{licenseId}",
        "options",
    ): [{"trapi.platform.license-management"}],
    ("/license-management/v2/api/welcome-emails/{welcomeEmailId}", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/welcome-emails/{welcomeEmailId}", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v2/api/welcome-emails/{welcomeEmailId}", "patch"): [
        {"trapi.platform.license-management"}
    ],
    ("/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/lifecycle-transitions/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/metadata-store/beta1/metadata-ids/{domain}/{subdomain}/{businessId}", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/metadata-ids/{domain}/{subdomain}/{businessId}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshot-status/{domain}/{subdomain}/{environment}/{jobId}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshot-status/{domain}/{subdomain}/{environment}/{jobId}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshot-status/{domain}/{subdomain}/{jobId}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshot-status/{domain}/{subdomain}/{jobId}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-snapshots/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/metadata-store/beta1/metadata-writes/{domain}/{subdomain}/status", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata-writes/{domain}/{subdomain}/status", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/metadata-writes/{domain}/{subdomain}/{environment}/status/{jobId}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-writes/{domain}/{subdomain}/{environment}/status/{jobId}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-writes/{domain}/{subdomain}/{environment}/{jobId}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata-writes/{domain}/{subdomain}/{environment}/{jobId}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}", "put"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}", "delete"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}", "put"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}", "delete"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}", "put"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}", "delete"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/metadata/{domain}/{subdomain}/{node1}/{node2}/{node3}/{node4}/{node5}/{node6}/{node7}/{node8}/{node9}/{node10}/{node11}/{node12}/{node13}/{node14}/{node15}/{node16}/{node17}/{node18}/{node19}/{node20}/{node21}/{node22}/{node23}/{node24}/{node25}/{node26}/{node27}/{node28}/{node29}/{node30}/{node31}/{node32}/{node33}/{node34}/{node35}/{node36}/{node37}/{node38}/{node39}/{node40}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/metadata-store/beta1/schemas/{domain}/{subdomain}", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/schemas/{domain}/{subdomain}", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/schemas/{domain}/{subdomain}/{schemaVersion}", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/schemas/{domain}/{subdomain}/{schemaVersion}", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/search/general", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/search/general", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/search/ui/any", "post"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/search/ui/any", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/storage/domains", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    ("/metadata-store/beta1/storage/domains", "options"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rule-writes/{jobId}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rule-writes/{jobId}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules", "get"): [
        {"trapi.metadata.read", "trapi.metadata.write"}
    ],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules",
        "post",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules/{ruleName}",
        "get",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules/{ruleName}",
        "put",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules/{ruleName}",
        "delete",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    (
        "/metadata-store/beta1/validations/reference/{domain}/{subdomain}/rules/{ruleName}",
        "options",
    ): [{"trapi.metadata.read", "trapi.metadata.write"}],
    ("/discovery/symbology/v1/convert", "options"): [],
    ("/discovery/symbology/v1/convert", "post"): [],
    ("/discovery/symbology/v1/debug", "options"): [],
    ("/discovery/symbology/v1/debug", "post"): [],
    ("/discovery/symbology/v1/health", "get"): [],
    ("/discovery/symbology/v1/health", "options"): [],
    ("/discovery/symbology/v1/lookup", "options"): [],
    ("/discovery/symbology/v1/lookup", "post"): [],
    ("/discovery/symbology/v1/lookup-type", "options"): [],
    ("/discovery/symbology/v1/lookup-type", "post"): [],
    ("/iam/access-control/v1/api/check", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/check", "post"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/check", "options"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/healthcheck", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/healthcheck", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/access-control/v1/api/permissions", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/permissions", "post"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/permissions", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/access-control/v1/api/resources", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/access-control/v1/api/resources", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/data/historical-pricing/beta1/views/events/{universe}", "get"): [
        {"trapi.data.historical-pricing.events.read"}
    ],
    ("/data/historical-pricing/beta1/views/events/{universe}", "options"): [],
    ("/data/historical-pricing/beta1/views/single-event/{universe}", "get"): [
        {"trapi.data.historical-pricing.events.read"}
    ],
    ("/data/historical-pricing/beta1/views/single-event/{universe}", "options"): [],
    ("/data/historical-pricing/beta1/views/summaries/{universe}", "get"): [
        {"trapi.data.historical-pricing.summaries.read"}
    ],
    ("/data/historical-pricing/beta1/views/summaries/{universe}", "options"): [],
    ("/iam/product-creation-and-maintenance/v1/exports", "post"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/exports", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/exports/{id}/output", "get"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/exports/{id}/output", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/exports/{id}/status", "get"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/exports/{id}/status", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/health", "get"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/health", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    (
        "/iam/product-creation-and-maintenance/v1/permission-sets/{technicalProductId}",
        "get",
    ): [{"trapi.platform.iam.product-maintenance"}],
    (
        "/iam/product-creation-and-maintenance/v1/permission-sets/{technicalProductId}",
        "options",
    ): [{"trapi.platform.iam.product-maintenance"}],
    ("/iam/product-creation-and-maintenance/v1/searches/tickets", "post"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/searches/tickets", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/tickets", "post"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/tickets", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/tickets/{id}", "get"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/tickets/{id}", "put"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/tickets/{id}", "options"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/iam/product-creation-and-maintenance/v1/tickets/{id}", "patch"): [
        {"trapi.platform.iam.product-maintenance"}
    ],
    ("/data/symbology/beta1/convert", "get"): [{"trapi.data.symbology.read"}],
    ("/data/symbology/beta1/convert", "post"): [{"trapi.data.symbology.read"}],
    ("/data/symbology/beta1/convert", "options"): [{"trapi.data.symbology.read"}],
    ("/user-framework/application-metadata/v1/aaaServiceSearch", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/aaaServiceSearch", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/appstore/aliases/metadata/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/aliases/metadata/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/check-management-access",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/check-management-access",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata/{appMdsId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata/{appMdsId}",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata/{appMdsId}/versions/latest",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata/{appMdsId}/versions/latest",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata/{appMdsId}/versions/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/application/metadata/{appMdsId}/versions/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/applications/metadata/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/applications/metadata/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/check",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/uploadStatus/{fileName}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/uploadStatus/{fileName}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/uploadUrl/{fileName}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/uploadUrl/{fileName}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/{fileHash}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/fileassets/{fileHash}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/package/release/{releaseMdsId}/approval-callback",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/package/release/{releaseMdsId}/approval-callback",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/release/{releaseMdsId}/approval-callback",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/release/{releaseMdsId}/approval-callback",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/search-engine/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/search-engine/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/service/version/{versionMdsId}/deployment",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/service/version/{versionMdsId}/deployment",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/metadata",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/metadata",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/ace/{aceId}",
        "delete",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/ace/{aceId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/ace/{aceId}",
        "patch",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/aces",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/aces",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/fileassets",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/fileassets",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/fileassets/prepare-download",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/fileassets/prepare-download",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/release",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/release",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/release",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/version/{versionMdsId}/release",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/versions/metadata/compare/{sourceMdsId}/{targetMdsId}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/appstore/versions/metadata/compare/{sourceMdsId}/{targetMdsId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/admin/config/static", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/config/static", "patch"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/config/static", "put"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/configuration/dynamic",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/admin/configuration/dynamic", "patch"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/configuration/static", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/configuration/static", "put"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/health/services", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/health/services", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/health/status", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/health/status", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/state-service/identifiers",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/state-service/identifiers",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/state-service/identifiers/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/state-service/identifiers/{entityIdentifierId}",
        "delete",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/state-service/identifiers/{entityIdentifierId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/state-service/identifiers/{entityIdentifierId}",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/application/{appMdsId}/scoped-groups",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/application/{appMdsId}/scoped-groups",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/admin/targeting/entities", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/targeting/entities", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/targeting/entities/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/entities/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/entities/users/count",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/entities/users/count",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/admin/targeting/group", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/admin/targeting/group", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}",
        "delete",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}/ace/{aceId}",
        "delete",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}/ace/{aceId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}/ace/{aceId}",
        "patch",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}/aces",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}/aces",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/group/{groupId}/aces",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/admin/targeting/groups/global", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/targeting/groups/global",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/groups/global/{globalGroupId}/management",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/groups/global/{globalGroupId}/management",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/groups/{groupId}/targetedgroups",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/admin/targeting/groups/{groupId}/targetedgroups",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/admin/targeting/user/status", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/admin/targeting/user/status",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/aliases", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/aliases", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/application", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/application", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/application/alias", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/application/version", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/application/version/{VersionMdsId}",
        "delete",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/application/version/{VersionMdsId}/fileassets",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/application/version/{VersionMdsId}/fileassets",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/application/version/{VersionMdsId}/fileassetsurl",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/application/version/{VersionMdsId}/fileassetsurl",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/application/{AppMdsId}", "delete"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/application/{AppMdsId}", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/application/{AppMdsId}", "put"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/application/{AppMdsId}/getversions",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/application/{AppMdsId}/groups", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/application/{AppMdsId}/groups",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/application/{AppMdsId}/versions",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/application/{AppMdsId}/versions",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/applications", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/applications", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/approval/request", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/approval/request", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/approval/{releaseMdsId}/callback",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/approval/{releaseMdsId}/callback",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/checkservices", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/checkservices", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/fileassets/check", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/fileassets/uploadurl/{FileName}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/fileassets/uploadurl/{FileName}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/fileassets/virusscan/{FileName}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/fileassets/virusscan/{FileName}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/fileassets/{FileHash}", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/fileassets/{FileHash}", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/getappslist", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/getappslist", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/getversionslist", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/getversionslist", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/globaldata", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/globaldata", "put"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}", "delete"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}/ace/{Operation}", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}/ace/{Operation}", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}/aces", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}/aces", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/group/{Id}/aces", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/groups/global", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/groups/global", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/groups/global/{Id}/management", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/groups/global/{Id}/management",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/groups/{Id}/targetedgroups", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/groups/{Id}/targetedgroups", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/kibana", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/kibana", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/resolve", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/resolve", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/stacks/getlist", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/stacks/getlist", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/stacks/{appMdsId}/versions/{versionMdsId}/nodes/{nodeMdsId}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/stacks/{appMdsId}/versions/{versionMdsId}/nodes/{nodeMdsId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/targeting/entities/get", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/targeting/entities/get", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/targeting/entities/members/count",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/targeting/entities/members/count",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/users/admin/status", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/users/admin/status", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/version/{VersionMdsId}/ace/{Operation}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/version/{VersionMdsId}/ace/{Operation}",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/version/{VersionMdsId}/aces",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/version/{VersionMdsId}/aces", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/version/{VersionMdsId}/release", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/version/{VersionMdsId}/release",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/version/{VersionMdsId}/release/{RequiredReleaseStatus}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/version/{VersionMdsId}/release/{RequiredReleaseStatus}",
        "put",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/versions/compare/{SourceMdsId}/{TargetMdsId}",
        "get",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/versions/compare/{SourceMdsId}/{TargetMdsId}",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/web/entitlements/apps/search", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/web/entitlements/apps/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/web/entitlements/apps/search", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/web/entitlements/resolve", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/web/entitlements/resolve", "post"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    (
        "/user-framework/application-metadata/v1/web/entitlements/versions/search",
        "options",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    (
        "/user-framework/application-metadata/v1/web/entitlements/versions/search",
        "post",
    ): [{"trapi.user-framework.application-metadata.raplib"}],
    ("/user-framework/application-metadata/v1/web/health/services", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/web/health/services", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/web/health/status", "get"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/application-metadata/v1/web/health/status", "options"): [
        {"trapi.user-framework.application-metadata.raplib"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-company-sector", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-company-sector", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-event-types", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-event-types", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-events-details", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-events-details", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-indices-list", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-indices-list", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-metadata", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-metadata", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-research-purchase", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-research-purchase", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-spark-line", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp-spark-line", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/benchmarks/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/benchmarks/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/business-classification/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/business-classification/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/business-summary/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/business-summary/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/capital-structure/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/capital-structure/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/company-name/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/company-name/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-ext-private/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-ext-private/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-ext-public/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-ext-public/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-private/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-private/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-public/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-info-public/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-structure/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/corporate-structure/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/bonds/tearsheet/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/bonds/tearsheet/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/equity/tearsheet/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/equity/tearsheet/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/loan/tearsheet/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/loan/tearsheet/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/merge-and-acquisition/tearsheet/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/deals/merge-and-acquisition/tearsheet/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/deals/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/deals/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/events/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/events/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/filings/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/filings/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/financial-performance/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/financial-performance/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/financial-summary/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/financial-summary/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/fundamental-ratio/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/fundamental-ratio/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/key-ratio-ext/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/key-ratio-ext/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/key-ratio/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/key-ratio/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/officer-and-directors-private/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/officer-and-directors-private/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/officer-and-directors-public/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/officer-and-directors-public/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/ownership/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/ownership/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/peers-ebitda-margin/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/peers-ebitda-margin/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/peers-ev/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/peers-ev/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/peers-ext/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/peers-ext/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/peers-marketcap/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/peers-marketcap/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/peers-percent-change/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/peers-percent-change/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/peers-revenue/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/corp/peers-revenue/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/price-performance-chart/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/price-performance-chart/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/price-performance-ext/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/price-performance-ext/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/price-performance/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/private-equity-investment/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/private-equity-investment/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/related-company/{permId}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/corp/related-company/{permId}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/corp/research/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/research/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/topinvestors/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/topinvestors/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/corp/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/fund/fund-performance/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/fund/fund-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/fund/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/fund/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/fund/industry-sectors/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/fund/industry-sectors/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/fund/investment-countries/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/fund/investment-countries/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/fund/overview/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/fund/overview/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/fund/price-performance/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/fund/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/fund/top-holdings/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/fund/top-holdings/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/general-asset/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/general-asset/healthcheck",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/general-assets/overview/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/general-assets/overview/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/general-assets/price-performance-chart/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/general-assets/price-performance-chart/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/general-assets/price-performance/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/general-assets/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/index/bottom-movers/index-point/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/bottom-movers/index-point/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/bottom-movers/percent-change/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/bottom-movers/percent-change/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/index/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/index/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/index/overview/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/overview-service/v1/index/overview/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/index/price-performance-chart/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/price-performance-chart/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/price-performance/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/top-movers/index-point/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/top-movers/index-point/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/top-movers/percent-change/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/overview-service/v1/index/top-movers/percent-change/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/overview-service/v1/private-company/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/overview-service/v1/private-company/healthcheck",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/data/symbology/duns-lookup/v1", "get"): [{"trapi.data.symbology.duns.read"}],
    ("/data/symbology/duns-lookup/v1", "options"): [{"trapi.data.symbology.duns.read"}],
    ("/streaming/trading-analytics/trade-data/beta1/redi", "get"): [
        {"trapi.streaming.trading-analytics.tds.stream.redi"}
    ],
    ("/streaming/trading-analytics/trade-data/beta1/redi", "options"): [
        {"trapi.streaming.trading-analytics.tds.stream.redi"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/fund-performance/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/fund-performance/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/industry-sectors/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/industry-sectors/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/investment-countries/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/fund-service/v1/fund/investment-countries/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/fund-service/v1/fund/overview/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/overview/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/price-performance/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/fund-service/v1/fund/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/fund-service/v1/fund/top-holdings/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/fund/top-holdings/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fund-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/ownership/beta1/views/consolidated/breakdown", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/breakdown", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/investors", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/investors", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/recent-activity", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/recent-activity", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/shareholders-history-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    (
        "/data/ownership/beta1/views/consolidated/shareholders-history-report",
        "options",
    ): [{"trapi.data.ownership.basic"}, {"trapi.data.ownership.adv"}],
    ("/data/ownership/beta1/views/consolidated/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/top-n-concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/consolidated/top-n-concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/breakdown", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/breakdown", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/holdings", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/holdings", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/investors", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/investors", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/recent-activity", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/recent-activity", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/shareholders-history-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/shareholders-history-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/top-n-concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/fund/top-n-concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/insider/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/insider/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/insider/transaction-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/insider/transaction-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/investor/holdings", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/investor/holdings", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/org-info", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/beta1/views/org-info", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/environmental-social-governance/v1/universe", "get"): [
        {"trapi.data.esg.universe.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/universe", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/basic", "get"): [
        {"trapi.data.esg.views-basic.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/basic", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/measures-full", "get"): [
        {"trapi.data.esg.views-measures-full.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/measures-full", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/measures-standard", "get"): [
        {"trapi.data.esg.views-measures-standard.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/measures-standard", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/scores-full", "get"): [
        {"trapi.data.esg.views-scores-full.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/scores-full", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/scores-standard", "get"): [
        {"trapi.data.esg.views-scores-standard.read", "trapi.data.esg.read"}
    ],
    ("/data/environmental-social-governance/v1/views/scores-standard", "options"): [
        {"trapi.data.esg.read"}
    ],
    ("/data/company-fundamentals/beta1/views/balance-sheet-brief/as-reported", "get"): [
        {"trapi.data.3pri.test.read"}
    ],
    (
        "/data/company-fundamentals/beta1/views/balance-sheet-brief/as-reported",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/balance-sheet-brief/standardized",
        "get",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/balance-sheet-brief/standardized",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    ("/data/company-fundamentals/beta1/views/cash-flow-brief/as-reported", "get"): [
        {"trapi.data.3pri.test.read"}
    ],
    ("/data/company-fundamentals/beta1/views/cash-flow-brief/as-reported", "options"): [
        {"trapi.data.3pri.test.read"}
    ],
    ("/data/company-fundamentals/beta1/views/cash-flow-brief/standardized", "get"): [
        {"trapi.data.3pri.test.read"}
    ],
    (
        "/data/company-fundamentals/beta1/views/cash-flow-brief/standardized",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    ("/data/company-fundamentals/beta1/views/financial-summary-brief", "get"): [
        {"trapi.data.3pri.test.read"}
    ],
    ("/data/company-fundamentals/beta1/views/financial-summary-brief", "options"): [
        {"trapi.data.3pri.test.read"}
    ],
    (
        "/data/company-fundamentals/beta1/views/income-statement-brief/as-reported",
        "get",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/income-statement-brief/as-reported",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/income-statement-brief/standardized",
        "get",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/income-statement-brief/standardized",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/operating-metrics-brief/as-reported",
        "get",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/operating-metrics-brief/as-reported",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/operating-metrics-brief/standardized",
        "get",
    ): [{"trapi.data.3pri.test.read"}],
    (
        "/data/company-fundamentals/beta1/views/operating-metrics-brief/standardized",
        "options",
    ): [{"trapi.data.3pri.test.read"}],
    ("/data/formula-analytics/v1/formulas", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/formula-analytics/v1/formulas", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/formula-analytics/v1/formulas/{formulaId}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/formula-analytics/v1/formulas/{formulaId}", "put"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/formula-analytics/v1/formulas/{formulaId}", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/formula-analytics/v1/formulas/{formulaId}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/file-store/beta2/buckets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/buckets", "post"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/buckets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/buckets/{name}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/buckets/{name}", "put"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/buckets/{name}", "delete"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/buckets/{name}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/bulk-publish", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/bulk-publish", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/claim-check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/claim-check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}/status", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}/status", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}/status/{status}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}/status/{status}", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/file-sets/{id}/status/{status}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files/{id}/stream", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/files/{id}/stream", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/info/health", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/info/health", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/info/s3check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/info/s3check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/info/version", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/info/version", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages", "post"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages/{id}", "put"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/packages/{id}", "head"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.read",
            "trapi.cfs.publisher.write",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.setup.write",
            "trapi.cfs.publisher.read",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.claimcheck.read",
            "trapi.cfs.publisher.read",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.read",
            "trapi.cfs.claimcheck.write",
        },
    ],
    ("/file-store/beta2/permission/packages", "get"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta2/permission/packages", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/buckets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/buckets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/buckets/{name}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/buckets/{name}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/buckets/{name}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/buckets/{name}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/claim-check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/claim-check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}/status", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}/status/{status}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}/status/{status}", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/file-sets/{id}/status/{status}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files/{id}/stream", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/files/{id}/stream", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta4/version", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/data/pricing/feed-processor/beta1/builds", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/builds", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/builds/{buildId}", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/builds/{buildId}", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/deployments", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/deployments", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/deployments/{deploymentId}", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/deployments/{deploymentId}", "options"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/beta1/deployments/{deploymentId}/shutdown",
        "post",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/beta1/deployments/{deploymentId}/shutdown",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/beta1/feed-store/{venue}/fields-mapping", "post"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/beta1/feed-store/{venue}/fields-mapping",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/beta1/feed-store/{venue}/lookup-config", "post"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/beta1/feed-store/{venue}/lookup-config",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/beta1/feed-store/{venue}/venue-config", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/beta1/feed-store/{venue}/venue-config", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/iam/accounts/v1/api/accounts", "get"): [{"trapi.platform.iam.accounts"}],
    ("/iam/accounts/v1/api/accounts", "options"): [{"trapi.platform.iam.accounts"}],
    ("/iam/accounts/v1/api/accounts/users/{uuid}/csln", "get"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/users/{uuid}/csln", "options"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/users/{uuid}/profile", "get"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/users/{uuid}/profile", "options"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}", "get"): [{"trapi.platform.iam.accounts"}],
    ("/iam/accounts/v1/api/accounts/{id}", "options"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}/associated-accounts", "get"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}/associated-accounts", "options"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}/cjr", "get"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}/cjr", "options"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}/users", "get"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/iam/accounts/v1/api/accounts/{id}/users", "options"): [
        {"trapi.platform.iam.accounts"}
    ],
    ("/messenger/beta1/chatrooms", "get"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/invites", "get"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}", "delete"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/invite", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/invite", "options"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/chatrooms/{chatroomId}/join", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/join", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/leave", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/leave", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/participants", "get"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/chatrooms/{chatroomId}/participants", "options"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/chatrooms/{chatroomId}/participants/remove", "post"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/chatrooms/{chatroomId}/participants/remove", "options"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/chatrooms/{chatroomId}/post", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/chatrooms/{chatroomId}/post", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/fileshare/upload", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/fileshare/upload", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/managed_chatrooms", "get"): [{"trapi.messenger"}],
    ("/messenger/beta1/managed_chatrooms", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/managed_chatrooms/{chatroomId}/join", "post"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/managed_chatrooms/{chatroomId}/join", "options"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/managed_chatrooms/{chatroomId}/leave", "post"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/managed_chatrooms/{chatroomId}/leave", "options"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/managed_chatrooms/{chatroomId}/post", "post"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/managed_chatrooms/{chatroomId}/post", "options"): [
        {"trapi.messenger"}
    ],
    ("/messenger/beta1/message", "post"): [{"trapi.messenger"}],
    ("/messenger/beta1/message", "options"): [{"trapi.messenger"}],
    ("/messenger/beta1/stream/events", "get"): [{"trapi.messenger"}],
    ("/messenger/beta1/stream/events", "options"): [{"trapi.messenger"}],
    ("/data/quantitative-analytics-curves-and-surfaces/beta1/surfaces", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-curves-and-surfaces/beta1/surfaces", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-curves-and-surfaces/beta1/surfaces_asl", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics-curves-and-surfaces/beta1/surfaces_asl",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/trading-analytics/reporting-services/beta1/fitca/jobs", "post"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/fitca/jobs", "options"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/fitca/jobs/{jobId}", "get"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/fitca/jobs/{jobId}",
        "options",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/fitca/upload-location",
        "post",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/fitca/upload-location",
        "options",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/rts27/jobs", "post"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/rts27/jobs", "options"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/rts27/jobs/{jobId}", "get"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/rts27/jobs/{jobId}",
        "options",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/rts27/upload-location",
        "post",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/rts27/upload-location",
        "options",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/rts28/jobs", "post"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/rts28/jobs", "options"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/trading-analytics/reporting-services/beta1/rts28/jobs/{jobId}", "get"): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/rts28/jobs/{jobId}",
        "options",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/rts28/upload-location",
        "post",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    (
        "/data/trading-analytics/reporting-services/beta1/rts28/upload-location",
        "options",
    ): [
        {
            "trapi.data.trading-analytics.reporting.rts28",
            "trapi.data.trading-analytics.reporting.rts27",
            "trapi.data.trading-analytics.reporting.fitca",
        }
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/AddPeriods", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/AddPeriods", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/add-periods", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/add-periods", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/count-periods", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics-dates-and-calendars/beta1/count-periods",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/date-schedule", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics-dates-and-calendars/beta1/date-schedule",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/holidays", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/holidays", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/holidays", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/is-working-day", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/beta1/is-working-day", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics-dates-and-calendars/beta1/is-working-day",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/symbology/duns-lookup/v2", "get"): [{"trapi.data.symbology.duns.read"}],
    ("/data/symbology/duns-lookup/v2", "options"): [{"trapi.data.symbology.duns.read"}],
    ("/user-framework/mobile/feedback-service/v1/feedback", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/feedback", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/feedback", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/issue", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/issue", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/upload-url", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/feedback-service/v1/upload-url", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/asset-type",
        "post",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/asset-type",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/benchmarks/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/benchmarks/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/datasource/{datasource}",
        "post",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/datasource/{datasource}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/interactive-chart/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/interactive-chart/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/issuer/rating/{identifier}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/issuer/rating/{identifier}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/organisation/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/organisation/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/overview/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/overview/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/price-performance/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/sparkline-chart",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/sparkline-chart",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/tick-chart/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/general-asset-service/v1/general-asset/tick-chart/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/general-asset-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/general-asset-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/digest", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/digest", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/portfolio", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/portfolio", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/taxonomy", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/taxonomy", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/taxonomy/{type}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/news-digest-service/v1/taxonomy/{type}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/datagrid/beta1", "post"): [{"trapi.data.get.data.read"}],
    ("/data/datagrid/beta1", "options"): [{"trapi.data.get.data.read"}],
    ("/license-management/v1/api/healthcheck", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/healthcheck", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/healthcheck-query", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/healthcheck-query", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/license-entitlements", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/license-entitlements", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/license-entitlements", "delete"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/license-entitlements", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/license-entitlements/{licenseEntitlementId}", "get"): [
        {"trapi.platform.license-management"}
    ],
    (
        "/license-management/v1/api/license-entitlements/{licenseEntitlementId}",
        "options",
    ): [{"trapi.platform.license-management"}],
    ("/license-management/v1/api/licenses", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/licenses", "post"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/licenses", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/licenses/{licenseId}", "get"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/licenses/{licenseId}", "put"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/licenses/{licenseId}", "delete"): [
        {"trapi.platform.license-management"}
    ],
    ("/license-management/v1/api/licenses/{licenseId}", "options"): [
        {"trapi.platform.license-management"}
    ],
    ("/trading/callouts-demo/beta1/fixed-income/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/fixed-income/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/fixed-income/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/irs/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/irs/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/irs/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/money-market/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/money-market/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/money-market/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/repo/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/repo/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/trading/callouts-demo/beta1/repo/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.demo"}
    ],
    ("/user-framework/mobile/index-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/index-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/index-service/v1/index/bottom-movers/index-point/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/bottom-movers/index-point/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/bottom-movers/percent-change/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/bottom-movers/percent-change/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/index-service/v1/index/overview/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/index-service/v1/index/overview/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/index-service/v1/index/price-performance/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/index-service/v1/index/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/top-movers/index-point/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/top-movers/index-point/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/top-movers/percent-change/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/index-service/v1/index/top-movers/percent-change/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/subscription-service/v1/device-registrations", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/devices/snooze-push-notification",
        "put",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/devices/snooze-push-notification",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/devices/snooze-push-notification/breaking-news",
        "put",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/devices/snooze-push-notification/breaking-news",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/{id}",
        "put",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/{id}",
        "delete",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/{id}/snooze",
        "put",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/device-registrations/{id}/snooze",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/subscription-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/subscription-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/subscription-service/v1/subscriptions", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/subscription-service/v1/subscriptions", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/subscription-service/v1/subscriptions", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/details/trigger/rics",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/details/trigger/rics",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/exists/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/exists/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/pause/bulk",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/pause/bulk",
        "patch",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/preferences/read",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/preferences/read",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/resume/bulk",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/subscription-service/v1/subscriptions/resume/bulk",
        "patch",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/subscription-service/v1/subscriptions/{id}", "put"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/subscription-service/v1/subscriptions/{id}", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/subscription-service/v1/subscriptions/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/iam/groups/v1/api/groups", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups", "post"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups", "options"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}", "put"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}", "delete"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}", "options"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}/members", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}/members", "post"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/groups/{id}/members", "delete"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/groups/v1/api/groups/{id}/members", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/groups/v1/api/system/healthcheck", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/groups/v1/api/system/healthcheck", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/data/custom-instruments/v1/events/{universe}", "get"): [
        {"trapi.data.historical-pricing.events.read"}
    ],
    ("/data/custom-instruments/v1/events/{universe}", "options"): [],
    ("/data/custom-instruments/v1/instrument-metadata", "get"): [
        {"trapi.synthetic.crud"}
    ],
    ("/data/custom-instruments/v1/instrument-metadata", "options"): [],
    ("/data/custom-instruments/v1/instruments", "post"): [{"trapi.synthetic.crud"}],
    ("/data/custom-instruments/v1/instruments", "options"): [],
    ("/data/custom-instruments/v1/instruments/{universe}", "get"): [
        {"trapi.synthetic.crud"}
    ],
    ("/data/custom-instruments/v1/instruments/{universe}", "put"): [
        {"trapi.synthetic.crud"}
    ],
    ("/data/custom-instruments/v1/instruments/{universe}", "delete"): [
        {"trapi.synthetic.crud"}
    ],
    ("/data/custom-instruments/v1/instruments/{universe}", "options"): [],
    ("/data/custom-instruments/v1/interday-summaries/{universe}", "get"): [
        {"trapi.data.historical-pricing.summaries.read"}
    ],
    ("/data/custom-instruments/v1/interday-summaries/{universe}", "options"): [],
    ("/data/custom-instruments/v1/intraday-summaries/{universe}", "get"): [
        {"trapi.data.historical-pricing.summaries.read"}
    ],
    ("/data/custom-instruments/v1/intraday-summaries/{universe}", "options"): [],
    ("/data/custom-instruments/v1/search", "get"): [{"trapi.synthetic.crud"}],
    ("/data/custom-instruments/v1/search", "options"): [],
    ("/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/accounts", "get"): [
        {"trapi.database-subscriptions.users.crud"}
    ],
    (
        "/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/accounts",
        "options",
    ): [{"trapi.database-subscriptions.users.crud"}],
    ("/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/users", "get"): [
        {"trapi.database-subscriptions.users.crud"}
    ],
    ("/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/users", "put"): [
        {"trapi.database-subscriptions.users.crud"}
    ],
    ("/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/users", "post"): [
        {"trapi.database-subscriptions.users.crud"}
    ],
    ("/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/users", "delete"): [
        {"trapi.database-subscriptions.users.crud"}
    ],
    ("/auth/cloud-access/beta1/retrieval-stores/{retrievalstoreId}/users", "options"): [
        {"trapi.database-subscriptions.users.crud"}
    ],
    ("/data/muni-pricing-modeling/v1/adjustments", "post"): [],
    ("/data/muni-pricing-modeling/v1/adjustments", "options"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/audit", "get"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/audit", "options"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/history", "get"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/history", "options"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/pricing", "post"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/pricing", "options"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/reason-codes", "get"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/reason-codes", "options"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/search", "post"): [],
    ("/data/muni-pricing-modeling/v1/adjustments/search", "options"): [],
    ("/data/muni-pricing-modeling/v1/benchmarks", "get"): [],
    ("/data/muni-pricing-modeling/v1/benchmarks", "post"): [],
    ("/data/muni-pricing-modeling/v1/benchmarks", "options"): [],
    ("/data/muni-pricing-modeling/v1/benchmarks/today", "get"): [],
    ("/data/muni-pricing-modeling/v1/benchmarks/today", "options"): [],
    ("/data/muni-pricing-modeling/v1/msrb", "post"): [],
    ("/data/muni-pricing-modeling/v1/msrb", "options"): [],
    ("/data/muni-pricing-modeling/v1/newissue", "get"): [],
    ("/data/muni-pricing-modeling/v1/newissue", "options"): [],
    ("/data/muni-pricing-modeling/v1/ontheflycalculations", "put"): [],
    ("/data/muni-pricing-modeling/v1/ontheflycalculations", "post"): [],
    ("/data/muni-pricing-modeling/v1/ontheflycalculations", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios", "get"): [],
    ("/data/muni-pricing-modeling/v1/portfolios", "put"): [],
    ("/data/muni-pricing-modeling/v1/portfolios", "post"): [],
    ("/data/muni-pricing-modeling/v1/portfolios", "delete"): [],
    ("/data/muni-pricing-modeling/v1/portfolios", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/adjustment", "post"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/adjustment", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/benchmark-rating", "get"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/benchmark-rating", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/bonds", "post"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/bonds", "delete"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/bonds", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/evaluators", "get"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/evaluators", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/grid-data", "get"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/grid-data", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/list", "get"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/list", "options"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/spread-constituents", "get"): [],
    ("/data/muni-pricing-modeling/v1/portfolios/spread-constituents", "options"): [],
    ("/data/muni-pricing-modeling/v1/sectors", "get"): [],
    ("/data/muni-pricing-modeling/v1/sectors", "post"): [],
    ("/data/muni-pricing-modeling/v1/sectors", "options"): [],
    ("/data/muni-pricing-modeling/v1/sectors/spread-constituents", "get"): [],
    ("/data/muni-pricing-modeling/v1/sectors/spread-constituents", "options"): [],
    ("/data/muni-pricing-modeling/v1/tolerance", "post"): [],
    ("/data/muni-pricing-modeling/v1/tolerance", "options"): [],
    ("/data/muni-pricing-modeling/v1/tolerance", "patch"): [],
    ("/data/muni-pricing-modeling/v1/tolerance/configuration", "get"): [],
    ("/data/muni-pricing-modeling/v1/tolerance/configuration", "put"): [],
    ("/data/muni-pricing-modeling/v1/tolerance/configuration", "options"): [],
    ("/data-store/document/v1/FinancialFiling/FilingDocument/{id}/{filename}", "get"): [
        {"trapi.graphql.subscriber.access"}
    ],
    (
        "/data-store/document/v1/FinancialFiling/FilingDocument/{id}/{filename}",
        "options",
    ): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/document/v1/version", "get"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/document/v1/version", "options"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/curve-chart/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/curve-chart/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/matching/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/matching/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/outrights/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/outrights/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/rates/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/forwards/rates/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/overview/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/overview/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/spot/change-summary/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/spot/change-summary/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/spot/price-performance/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/fx-service/v1/fx/spot/price-performance/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/fx-service/v1/fx/spot/price/{ric}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/fx/spot/price/{ric}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/fx-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/alerts/v1/news-headlines-subscriptions", "get"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/v1/news-headlines-subscriptions", "post"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/v1/news-headlines-subscriptions", "delete"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/v1/news-headlines-subscriptions", "options"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/v1/news-stories-subscriptions", "get"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/alerts/v1/news-stories-subscriptions", "post"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/alerts/v1/news-stories-subscriptions", "delete"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/alerts/v1/news-stories-subscriptions", "options"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/order-rights-management/content-services/v1/api/content-services", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/content-services", "post"): [
        {"rdpapi.data.csa.write"}
    ],
    (
        "/order-rights-management/content-services/v1/api/content-services",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/content-services/{contentServiceId}/segments/{contentSegmentId}",
        "delete",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{contentServiceId}/segments/{contentSegmentId}",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/content-services/{id}", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}",
        "delete",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/client-rules",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/client-rules",
        "patch",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/gx-model",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/gx-model",
        "patch",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/segments",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/segments",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/segments",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/segments",
        "patch",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/status",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/content-services/{id}/status",
        "patch",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/entitlements-report/supplier-service/{supplierServiceId}",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/entitlements-report/supplier-service/{supplierServiceId}",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/entitlements-report/{reportId}",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/entitlements-report/{reportId}",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/feature-toggles", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/feature-toggles", "options"): [],
    ("/order-rights-management/content-services/v1/api/gx-model", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/gx-model", "options"): [],
    ("/order-rights-management/content-services/v1/api/gx-model-mapping/{id}", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    (
        "/order-rights-management/content-services/v1/api/gx-model-mapping/{id}",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/gx-model-mapping/{id}",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/gx-model-mapping/{id}/content-service",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/gx-model-mapping/{id}/content-service",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/gx-model/{id}", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/gx-model/{id}", "options"): [],
    ("/order-rights-management/content-services/v1/api/products", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/products", "options"): [],
    ("/order-rights-management/content-services/v1/api/report", "post"): [
        {"rdpapi.data.csa.write"}
    ],
    ("/order-rights-management/content-services/v1/api/report", "options"): [],
    (
        "/order-rights-management/content-services/v1/api/report/status/{fileKey}",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/report/status/{fileKey}",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/resource-group", "post"): [
        {"rdpapi.data.csa.write"}
    ],
    ("/order-rights-management/content-services/v1/api/resource-group", "options"): [],
    (
        "/order-rights-management/content-services/v1/api/resource-group/{supplierServiceId}/{contentSegmentId}",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/resource-group/{supplierServiceId}/{contentSegmentId}",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/ss/gx-model", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/ss/gx-model", "options"): [],
    (
        "/order-rights-management/content-services/v1/api/ss/gx-model-mapping/{id}",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/ss/gx-model-mapping/{id}",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/ss/gx-model-mapping/{id}",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/ss/gx-model-mapping/{supplierServiceId}/supplier-service",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/ss/gx-model-mapping/{supplierServiceId}/supplier-service",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/ss/gx-model/{id}", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    (
        "/order-rights-management/content-services/v1/api/ss/gx-model/{id}",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/supplier-services", "get"): [
        {"rdpapi.data.csa.write"}
    ],
    (
        "/order-rights-management/content-services/v1/api/supplier-services",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/list",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/list",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/listNew",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/listNew",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/search",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/search",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/entitlements",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/entitlements",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/gx-model",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/gx-model",
        "patch",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/info",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/info",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/segments",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/segments",
        "post",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/segments",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/segments/{contentSegmentId}",
        "delete",
    ): [{"rdpapi.data.csa.write"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/ss/{supplierServiceId}/segments/{contentSegmentId}",
        "options",
    ): [],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/v2/ss/{supplierServiceId}/segments",
        "get",
    ): [{"rdpapi.data.csa.read"}],
    (
        "/order-rights-management/content-services/v1/api/supplier-services/v2/ss/{supplierServiceId}/segments",
        "options",
    ): [],
    ("/order-rights-management/content-services/v1/api/user-info", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/api/user-info", "options"): [],
    ("/order-rights-management/content-services/v1/health", "get"): [
        {"rdpapi.data.csa.read"}
    ],
    ("/order-rights-management/content-services/v1/health", "options"): [],
    ("/data/pricing/feed-processor/v1/builds", "post"): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v1/builds", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/builds/{buildId}", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/builds/{buildId}", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/deployments", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/deployments", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/deployments/{deploymentId}", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/deployments/{deploymentId}", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/deployments/{deploymentId}/shutdown", "post"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/v1/deployments/{deploymentId}/shutdown",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v1/feed-store/{venue}/fields-mapping", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/feed-store/{venue}/fields-mapping", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/feed-store/{venue}/lookup-config", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/feed-store/{venue}/lookup-config", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/feed-store/{venue}/venue-config", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v1/feed-store/{venue}/venue-config", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/ownership/v1/views/consolidated/breakdown", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/breakdown", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/investors", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/investors", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/recent-activity", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/recent-activity", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/shareholders-history-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/shareholders-history-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/top-n-concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/consolidated/top-n-concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/breakdown", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/breakdown", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/holdings", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/holdings", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/investors", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/investors", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/recent-activity", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/recent-activity", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/shareholders-history-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/shareholders-history-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/top-n-concentration", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/fund/top-n-concentration", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/insider/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/insider/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/insider/transaction-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/insider/transaction-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/investor/holdings", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/investor/holdings", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/org-info", "get"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.basic", "trapi.demo-access"},
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
        {"trapi.demo-access", "trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/org-info", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/ultimateparent/shareholders-report", "get"): [
        {"trapi.data.ownership.basic", "trapi.data.ownership.adv"},
        {"trapi.data.ownership.adv"},
    ],
    ("/data/ownership/v1/views/ultimateparent/shareholders-report", "options"): [
        {"trapi.data.ownership.basic"},
        {"trapi.data.ownership.adv"},
    ],
    ("/user-framework/mobile/briefcase-service/v1/briefcase", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/briefcase-service/v1/briefcase", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/briefcase-service/v1/briefcase", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/briefcase-service/v1/briefcase", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/briefcase-service/v1/briefcase/mark-as-read", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/briefcase-service/v1/briefcase/mark-as-read", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/briefcase-service/v1/briefcase/mark-as-read/news",
        "post",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/briefcase-service/v1/briefcase/mark-as-read/news",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/briefcase-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/briefcase-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/partners/lusid/v1/aggregation/aggregation/$valuation", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/aggregation/aggregation/$valuation", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/aggregation/aggregation/$valuationinlined", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/aggregation/aggregation/$valuationinlined", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/aggregation/results/queryable/keys", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/aggregation/results/queryable/keys", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/credit/conventions", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/credit/conventions", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/credit/conventions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/credit/conventions/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/credit/conventions/{scope}/{code}", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/credit/conventions/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/flowconventions", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/flowconventions", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/flowconventions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/flowconventions/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/flowconventions/{scope}/{code}", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/partners/lusid/v1/conventions/rates/flowconventions/{scope}/{code}",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/conventions/rates/indexconventions", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/indexconventions", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/indexconventions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/conventions/rates/indexconventions/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/partners/lusid/v1/conventions/rates/indexconventions/{scope}/{code}",
        "delete",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/partners/lusid/v1/conventions/rates/indexconventions/{scope}/{code}",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/datatypes/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/datatypes/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/health", "get"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/health", "options"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/health/verify", "get"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/health/verify", "options"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/instrumentevents/$query", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$query", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$queryBucketedCashFlows", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$queryBucketedCashFlows", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$queryCashFlows", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$queryCashFlows", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$queryTradeTickets", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instrumentevents/$queryTradeTickets", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments", "get"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/instruments", "post"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/instruments", "options"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/instruments/$get", "post"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/instruments/$get", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments/identifierTypes", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments/identifierTypes", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments/{identifierType}/{identifier}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments/{identifierType}/{identifier}", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments/{identifierType}/{identifier}", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/instruments/{identifierType}/{identifier}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/partners/lusid/v1/instruments/{identifierType}/{identifier}/properties",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/partners/lusid/v1/instruments/{identifierType}/{identifier}/properties",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/partners/lusid/v1/instruments/{identifierType}/{identifier}/properties/list",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/partners/lusid/v1/instruments/{identifierType}/{identifier}/properties/list",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/metadata/versions", "get"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/metadata/versions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/propertydefinitions", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/propertydefinitions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/propertydefinitions/{domain}/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/propertydefinitions/{domain}/{scope}/{code}", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/propertydefinitions/{domain}/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/quotes/{scope}", "get"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/quotes/{scope}", "post"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/quotes/{scope}", "options"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/quotes/{scope}/$delete", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/quotes/{scope}/$delete", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/quotes/{scope}/$get", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/quotes/{scope}/$get", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/sequence/{scope}", "post"): [{"trapi.data.portfolios.read"}],
    ("/partners/lusid/v1/sequence/{scope}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/sequence/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/sequence/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/sequence/{scope}/{code}/increment", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/sequence/{scope}/{code}/increment", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/systemconfiguration/transactions", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/systemconfiguration/transactions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/translation/instrumentDefinitions", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/partners/lusid/v1/translation/instrumentDefinitions", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/data/analytics/models/beta1/equity/market-impact/impact", "options"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/equity/market-impact/impact", "post"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/equity/market-impact/models", "get"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/equity/market-impact/models", "options"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/equity/market-impact/rics", "get"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/equity/market-impact/rics", "options"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/equity/market-impact/volume-volatility",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/equity/market-impact/volume-volatility", "post"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/analytics/current/{model_type}/chart",
        "get",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/analytics/current/{model_type}/chart",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/analytics/current/{model_type}/distribution",
        "get",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/analytics/current/{model_type}/distribution",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/analytics/current/{model_type}/stats",
        "get",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/analytics/current/{model_type}/stats",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/assets/current/{model_type}",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/assets/current/{model_type}",
        "post",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/assets/current/{model_type}/filter",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/assets/current/{model_type}/filter",
        "post",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/coverage/{model_type}",
        "get",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/data/analytics/models/beta1/fixed-income/fi-scoring/coverage/{model_type}",
        "options",
    ): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/fixed-income/fi-scoring/models", "get"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    ("/data/analytics/models/beta1/fixed-income/fi-scoring/models", "options"): [
        {
            "trapi.data.ms-analytics.market-impact-param",
            "trapi.data.ms-analytics.bond-liquidity-starmine",
        }
    ],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/chart/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/chart/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/economic-event-service/v1/economic-event/events", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-filters-options",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-filters-options",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles-filter",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles-filter",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles-filter-indicators",
        "post",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles-filter-indicators",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles-filter/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events-profiles-filter/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events/{ric}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/events/{ric}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/performance",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/economic-event-service/v1/economic-event/performance",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/economic-event-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/economic-event-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/iam/client-sso-admin/v1/api/healthcheck", "get"): [{"trapi.platform.iam.sso"}],
    ("/iam/client-sso-admin/v1/projects", "get"): [{"trapi.platform.iam.sso"}],
    ("/iam/client-sso-admin/v1/projects", "post"): [{"trapi.platform.iam.sso"}],
    ("/iam/client-sso-admin/v1/projects", "options"): [{"trapi.platform.iam.sso"}],
    ("/iam/client-sso-admin/v1/projects/{projectId}", "get"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}", "put"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}", "delete"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}", "options"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/metadata", "get"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/metadata", "put"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/metadata", "options"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/metadata/cert", "put"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/metadata/cert", "options"): [
        {"trapi.platform.iam.sso"}
    ],
    (
        "/iam/client-sso-admin/v1/projects/{projectId}/metadata/idp-domain-whitelist",
        "put",
    ): [{"trapi.platform.iam.sso"}],
    (
        "/iam/client-sso-admin/v1/projects/{projectId}/metadata/idp-domain-whitelist",
        "options",
    ): [{"trapi.platform.iam.sso"}],
    (
        "/iam/client-sso-admin/v1/projects/{projectId}/metadata/sp-attribute-mappings",
        "put",
    ): [{"trapi.platform.iam.sso"}],
    (
        "/iam/client-sso-admin/v1/projects/{projectId}/metadata/sp-attribute-mappings",
        "options",
    ): [{"trapi.platform.iam.sso"}],
    ("/iam/client-sso-admin/v1/projects/{projectId}/ping-federate", "post"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/ping-federate", "delete"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/ping-federate", "options"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/ping-federate", "patch"): [
        {"trapi.platform.iam.sso"}
    ],
    ("/iam/client-sso-admin/v1/projects/{projectId}/ping-federate/metadata", "get"): [
        {"trapi.platform.iam.sso"}
    ],
    (
        "/iam/client-sso-admin/v1/references/ping-federate/authentication-policy-contracts",
        "get",
    ): [{"trapi.platform.iam.sso"}],
    (
        "/iam/client-sso-admin/v1/references/ping-federate/authentication-policy-contracts",
        "options",
    ): [{"trapi.platform.iam.sso"}],
    ("/iam/apps/v1/api/appaccounts", "get"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/appaccounts", "post"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/appaccounts", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/appaccounts/{id}", "get"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/appaccounts/{id}", "put"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/appaccounts/{id}", "delete"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/appaccounts/{id}", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/client-status/{uuid}", "get"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/client-status/{uuid}", "options"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/healthcheck", "get"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/healthcheck", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities", "post"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities-certify/{id}", "put"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/identities-certify/{id}", "options"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/identities-deregister/{id}", "options"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/identities-deregister/{id}", "patch"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/identities-details", "get"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities-details", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities-details/{id}", "get"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/identities-details/{id}", "options"): [
        {"trapi.platform.iam.app_id"}
    ],
    ("/iam/apps/v1/api/identities-map", "post"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities-map", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities/{id}", "get"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/identities/{id}", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/jwks", "put"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/jwks", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/password", "put"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/password", "options"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/search/identities", "post"): [{"trapi.platform.iam.app_id"}],
    ("/iam/apps/v1/api/search/identities", "options"): [{"trapi.platform.iam.app_id"}],
    ("/data/pricing/beta1/ppestreaming", "get"): [
        {"trapi.streaming.pricing.read"},
        {"trapi.data.pricing.read"},
    ],
    ("/data/pricing/beta1/ppestreaming", "options"): [
        {"trapi.streaming.pricing.read"},
        {"trapi.data.pricing.read"},
    ],
    ("/sentimine-and-summarization/v1/events", "get"): [{"trapi.frtb.sentimarization"}],
    ("/data/ownership/views/ultimate-beneficial-ownership-availability/v1", "get"): [
        {"trapi.data.ownership.ubo.read"}
    ],
    (
        "/data/ownership/views/ultimate-beneficial-ownership-availability/v1",
        "options",
    ): [{"trapi.data.ownership.ubo.read"}],
    ("/user-framework/mobile/user-service/v1/access", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/access", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/auth", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/auth", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/attributes", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/attributes", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/content-language", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/content-language", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/content-language", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/uuid", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/profile/uuid", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/support/call-center", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/user-service/v1/support/call-center", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/streaming/quantitative-analytics/beta1/financial-contracts", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/streaming/quantitative-analytics/beta1/financial-contracts", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/news/beta1/analyze", "get"): [{"trapi.data.news.read"}],
    ("/data/news/beta1/analyze", "options"): [{"trapi.data.news.read"}],
    ("/data/news/beta1/headlines", "get"): [{"trapi.data.news.read"}],
    ("/data/news/beta1/headlines", "options"): [{"trapi.data.news.read"}],
    ("/data/news/beta1/stories/{storyId}", "get"): [{"trapi.data.news.read"}],
    ("/data/news/beta1/stories/{storyId}", "options"): [{"trapi.data.news.read"}],
    ("/trading/idc-demo/deals/v2/reports", "get"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports", "options"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports/find-by-ticket", "get"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports/find-by-ticket", "options"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports/non-conventional-deal", "get"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports/non-conventional-deal", "options"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports/time-range", "get"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/trading/idc-demo/deals/v2/reports/time-range", "options"): [
        {"trapi.islamic-deal.dealreport.demo.access"}
    ],
    ("/api-lifecycle/beta1/apis", "get"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis", "options"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis", "post"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis/{apiId}", "delete"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis/{apiId}", "get"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis/{apiId}", "options"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis/{apiId}/artifacts/{artifactType}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/artifacts/{artifactType}", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/artifacts/{artifactType}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/detail", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/detail", "put"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/apis/{apiId}/gate/{gateId}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/gate/{gateId}", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/gate/{gateId}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/milestone/{milestoneId}", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/milestone/{milestoneId}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/milestone/{milestoneId}/activate", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/apis/{apiId}/milestone/{milestoneId}/activate", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/assetid/{assetId}", "get"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/assetid/{assetId}", "options"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/categories", "get"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/categories", "options"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}", "get"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}/attachment", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}/attachment", "post"): [
        {"trapi.deployment.crud"}
    ],
    (
        "/api-lifecycle/beta1/discussion/{apiId}/{gateId}/attachment/{attachmentId}",
        "delete",
    ): [{"trapi.deployment.crud"}],
    (
        "/api-lifecycle/beta1/discussion/{apiId}/{gateId}/attachment/{attachmentId}",
        "get",
    ): [{"trapi.deployment.crud"}],
    (
        "/api-lifecycle/beta1/discussion/{apiId}/{gateId}/attachment/{attachmentId}",
        "options",
    ): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}/comment", "options"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}/comment", "post"): [
        {"trapi.deployment.crud"}
    ],
    (
        "/api-lifecycle/beta1/discussion/{apiId}/{gateId}/comment/{commentId}",
        "delete",
    ): [{"trapi.deployment.crud"}],
    (
        "/api-lifecycle/beta1/discussion/{apiId}/{gateId}/comment/{commentId}",
        "options",
    ): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/discussion/{apiId}/{gateId}/comment/{commentId}", "put"): [
        {"trapi.deployment.crud"}
    ],
    ("/api-lifecycle/beta1/whoami", "get"): [{"trapi.deployment.crud"}],
    ("/api-lifecycle/beta1/whoami", "options"): [{"trapi.deployment.crud"}],
    ("/alerts/beta1/news-headlines-subscriptions", "get"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/beta1/news-headlines-subscriptions", "post"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/beta1/news-headlines-subscriptions", "delete"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/beta1/news-headlines-subscriptions", "options"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/alerts/beta1/news-stories-subscriptions", "get"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/alerts/beta1/news-stories-subscriptions", "post"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/alerts/beta1/news-stories-subscriptions", "delete"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/alerts/beta1/news-stories-subscriptions", "options"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/data/quantitative-analytics-voyage-rate/v1/maritime-freight-rate", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-voyage-rate/v1/maritime-freight-rate", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/message-services/beta1/news-readership/story/subscriptions", "get"): [
        {"trapi.msg-svcs.readership-story.crud"}
    ],
    ("/message-services/beta1/news-readership/story/subscriptions", "post"): [
        {"trapi.msg-svcs.readership-story.crud"}
    ],
    ("/message-services/beta1/news-readership/story/subscriptions", "delete"): [
        {"trapi.msg-svcs.readership-story.crud"}
    ],
    ("/message-services/beta1/news-readership/story/subscriptions", "options"): [
        {"trapi.msg-svcs.readership-story.crud"}
    ],
    ("/apiadmin/beta1/aggregate/definitions", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/aggregate/definitions", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/aggregate/definitions/{id}", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/aggregate/definitions/{id}", "options"): [
        {"trapi.data.api.test"}
    ],
    ("/apiadmin/beta1/approval", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/approval", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/approval/grant", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/approval/grant", "post"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/approval/require", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/approval/require", "post"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/connectivity", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/connectivity", "post"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/definitions", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/definitions", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/definitions/{id}", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/definitions/{id}", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/definitions/{id}/swagger", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/definitions/{id}/swagger", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/redirect", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/redirect", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/support", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta1/support", "options"): [{"trapi.data.api.test"}],
    ("/auth/cloud-credentials/beta1", "get"): [{"trapi.auth.cloud-credentials"}],
    ("/auth/cloud-credentials/beta1", "options"): [{"trapi.auth.cloud-credentials"}],
    ("/iam/users/v1/api/anytype/{uuid}", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/anytype/{uuid}", "options"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/anytype/{uuid}/relocation", "post"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/anytype/{uuid}/relocation", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/externalUsers/{id}/{product}", "get"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/externalUsers/{id}/{product}", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/externalUsers/{id}/{product}/{system}", "get"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/externalUsers/{id}/{product}/{system}", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/externalUsers/{id}/{product}/{system}/{subsystem}", "get"): [
        {"trapi.platform.iam.users"}
    ],
    (
        "/iam/users/v1/api/externalUsers/{id}/{product}/{system}/{subsystem}",
        "options",
    ): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/serviceAccounts", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/serviceAccounts", "post"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/serviceAccounts", "options"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/serviceAccounts/{id}", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/serviceAccounts/{id}", "put"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/serviceAccounts/{id}", "delete"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/serviceAccounts/{id}", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/serviceAccounts/{id}", "patch"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/system/healthcheck", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/system/healthcheck", "options"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/system/healthcheck/upstream", "get"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/system/healthcheck/upstream", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users", "put"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users", "post"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users", "options"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{id}", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{id}", "put"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{id}", "delete"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{id}", "options"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{id}", "patch"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{userId}/hard", "delete"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{userId}/hard", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId", "get"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}", "get"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}", "put"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}", "delete"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}", "options"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}", "get"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}", "put"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}", "delete"): [
        {"trapi.platform.iam.users"}
    ],
    ("/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}", "options"): [
        {"trapi.platform.iam.users"}
    ],
    (
        "/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}/{subsystem}",
        "get",
    ): [{"trapi.platform.iam.users"}],
    (
        "/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}/{subsystem}",
        "put",
    ): [{"trapi.platform.iam.users"}],
    (
        "/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}/{subsystem}",
        "delete",
    ): [{"trapi.platform.iam.users"}],
    (
        "/iam/users/v1/api/users/{uuid}/externalId/{product}/{system}/{subsystem}",
        "options",
    ): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{uuid}/ftl", "get"): [{"trapi.platform.iam.users"}],
    ("/iam/users/v1/api/users/{uuid}/ftl", "options"): [{"trapi.platform.iam.users"}],
    ("/docs/beta1/definitions", "get"): [],
    ("/docs/beta1/definitions", "options"): [],
    ("/docs/beta1/definitions/{id}", "get"): [],
    ("/docs/beta1/definitions/{id}", "options"): [],
    ("/docs/beta1/definitions/{id}/demoviews", "get"): [],
    ("/docs/beta1/definitions/{id}/demoviews", "options"): [],
    ("/docs/beta1/definitions/{id}/examples", "get"): [],
    ("/docs/beta1/definitions/{id}/examples", "options"): [],
    ("/docs/beta1/definitions/{id}/readme", "get"): [],
    ("/docs/beta1/definitions/{id}/readme", "options"): [],
    ("/data/pricing/feed-processor/v2/ami", "get"): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/ami", "options"): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/ami/approved", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/ami/approved", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/ami/approved", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/feeds", "get"): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/feeds", "options"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefact-context/fields-mapping",
        "put",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefact-context/fields-mapping",
        "options",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefact-context/lookup-config",
        "put",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefact-context/lookup-config",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefacts", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefacts", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefacts", "options"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/build-artefacts/{build_artefact_id}",
        "get",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/processors", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/processors", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/processors", "options"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors-context/venue-config",
        "put",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors-context/venue-config",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/processors-schedule", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/feeds/{feed_name}/processors-schedule", "post"): [
        {"trapi.data.feed-processor"}
    ],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors-schedule",
        "delete",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors-schedule",
        "options",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}",
        "delete",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}",
        "options",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}/redeploy",
        "post",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}/redeploy",
        "options",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}/status",
        "get",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}/status/post-install",
        "get",
    ): [{"trapi.data.feed-processor"}],
    (
        "/data/pricing/feed-processor/v2/feeds/{feed_name}/processors/{processor_id}/status/post-install",
        "options",
    ): [{"trapi.data.feed-processor"}],
    ("/data/pricing/feed-processor/v2/graphql", "post"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/graphql", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/processors-schedule", "get"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/pricing/feed-processor/v2/processors-schedule", "options"): [
        {"trapi.data.feed-processor"}
    ],
    ("/data/units/v1/measurement/conversion", "get"): [
        {
            "trapi.data.historical-pricing.summaries.read",
            "trapi.data.historical-pricing.events.read",
        }
    ],
    ("/data/units/v1/measurement/conversion", "options"): [],
    ("/data/units/v1/measurement/information", "get"): [
        {
            "trapi.data.historical-pricing.summaries.read",
            "trapi.data.historical-pricing.events.read",
        }
    ],
    ("/data/units/v1/measurement/information", "options"): [],
    ("/data/units/v1/measurement/information/{unitCode}", "get"): [
        {
            "trapi.data.historical-pricing.summaries.read",
            "trapi.data.historical-pricing.events.read",
        }
    ],
    ("/data/units/v1/measurement/information/{unitCode}", "options"): [],
    ("/trading/callouts/beta1/fixed-income/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/fixed-income/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/fixed-income/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/irs/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/irs/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/irs/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/money-market/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/money-market/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/money-market/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/repo/trade-capture-report", "get"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/repo/trade-capture-report", "post"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/trading/callouts/beta1/repo/trade-capture-report", "options"): [
        {"trapi.trading.callouts.posttrade.prod"}
    ],
    ("/auth/oauth2/v1/authorize", "get"): [],
    ("/auth/oauth2/v1/revoke", "post"): [],
    ("/auth/oauth2/v1/token", "post"): [],
    ("/trading/idc/deals/v2/reports", "get"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports", "options"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports/find-by-ticket", "get"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports/find-by-ticket", "options"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports/non-conventional-deal", "get"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports/non-conventional-deal", "options"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports/time-range", "get"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/trading/idc/deals/v2/reports/time-range", "options"): [
        {"trapi.islamic-deal.dealreport.access"}
    ],
    ("/data/pricing/feed-config/v1/channel-context-processor", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-context-processor", "post"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-context-processor", "options"): [
        {"trapi.data.feed-config"}
    ],
    (
        "/data/pricing/feed-config/v1/channel-context-processor/{channelContextLinkProcessorId}",
        "get",
    ): [{"trapi.data.feed-config"}],
    (
        "/data/pricing/feed-config/v1/channel-context-processor/{channelContextLinkProcessorId}",
        "delete",
    ): [{"trapi.data.feed-config"}],
    (
        "/data/pricing/feed-config/v1/channel-context-processor/{channelContextLinkProcessorId}",
        "options",
    ): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/channel-contextid-links", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-contextid-links", "post"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-contextid-links", "options"): [
        {"trapi.data.feed-config"}
    ],
    (
        "/data/pricing/feed-config/v1/channel-contextid-links/{channelContextLinkId}",
        "get",
    ): [{"trapi.data.feed-config"}],
    (
        "/data/pricing/feed-config/v1/channel-contextid-links/{channelContextLinkId}",
        "delete",
    ): [{"trapi.data.feed-config"}],
    (
        "/data/pricing/feed-config/v1/channel-contextid-links/{channelContextLinkId}",
        "options",
    ): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/channel-networks", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-networks", "post"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-networks", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-networks/{channelNetworkId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-networks/{channelNetworkId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-networks/{channelNetworkId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-schedules", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-schedules", "post"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-schedules", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-schedules/{channelScheduleId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-schedules/{channelScheduleId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channel-schedules/{channelScheduleId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channels", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/channels", "post"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/channels", "options"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/channels/{channelId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channels/{channelId}", "put"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channels/{channelId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/channels/{channelId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/closing-run-header", "post"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/closing-run-header", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/closing-run-header", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contentGroupContacts", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contentGroupContacts", "options"): [
        {"trapi.data.feed-config"}
    ],
    (
        "/data/pricing/feed-config/v1/contentGroupContacts/{contentGroupContactsId}",
        "get",
    ): [{"trapi.data.feed-config"}],
    (
        "/data/pricing/feed-config/v1/contentGroupContacts/{contentGroupContactsId}",
        "options",
    ): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/contextids", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/contextids", "post"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/contextids", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contextids/{contextId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contextids/{contextId}", "put"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contextids/{contextId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contextids/{contextId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/contextids/{contextId}/fidlist", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/data-models", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/data-models", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/data-models/{dataModelId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/data-models/{dataModelId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/feeds", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/feeds", "post"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/feeds", "options"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/feeds/{feedId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/feeds/{feedId}", "put"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/feeds/{feedId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/feeds/{feedId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/mas/mapping", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/mas/mapping", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/peContacts", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/peContacts", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/peContacts/{peContactsId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/peContacts/{peContactsId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/processors", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/processors", "post"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/processors", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/processors/{processorId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/processors/{processorId}", "put"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/processors/{processorId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/processors/{processorId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/services", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/services", "post"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/services", "options"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/services/{serviceId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/services/{serviceId}", "put"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/services/{serviceId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/services/{serviceId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/techOpsContacts", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/techOpsContacts", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/techOpsContacts/{techOpsContactsId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/techOpsContacts/{techOpsContactsId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/venues", "get"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/venues", "post"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/venues", "options"): [{"trapi.data.feed-config"}],
    ("/data/pricing/feed-config/v1/venues/{venueId}", "get"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/venues/{venueId}", "put"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/venues/{venueId}", "delete"): [
        {"trapi.data.feed-config"}
    ],
    ("/data/pricing/feed-config/v1/venues/{venueId}", "options"): [
        {"trapi.data.feed-config"}
    ],
    ("/user-framework/bold/beta2/knowledge-graph-score", "post"): [{"trapi.sdbold"}],
    ("/user-framework/bold/beta2/knowledge-graph-score", "options"): [{"trapi.sdbold"}],
    ("/user-framework/bold/beta2/knowledge-graph-score/{token}/result", "get"): [
        {"trapi.sdbold"}
    ],
    ("/user-framework/bold/beta2/knowledge-graph-score/{token}/result", "options"): [
        {"trapi.sdbold"}
    ],
    ("/user-framework/bold/beta2/knowledge-graph-score/{token}/status", "get"): [
        {"trapi.sdbold"}
    ],
    ("/user-framework/bold/beta2/knowledge-graph-score/{token}/status", "options"): [
        {"trapi.sdbold"}
    ],
    (
        "/user-framework/bold/beta2/knowledge-graph-score/{token}/summary/organization",
        "get",
    ): [{"trapi.sdbold"}],
    (
        "/user-framework/bold/beta2/knowledge-graph-score/{token}/summary/organization",
        "options",
    ): [{"trapi.sdbold"}],
    ("/user-framework/bold/beta2/knowledge-graph-score/{token}/summary/topic", "get"): [
        {"trapi.sdbold"}
    ],
    (
        "/user-framework/bold/beta2/knowledge-graph-score/{token}/summary/topic",
        "options",
    ): [{"trapi.sdbold"}],
    ("/user-framework/workspace/beta0", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/ap", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/ap", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/workspace/beta0/calendars/{calendarId}/secret-addresses",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/beta0/calendars/{calendarId}/secret-addresses",
        "post",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/beta0/calendars/{calendarId}/secret-addresses",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/beta0/calendars/{calendarId}/secret-addresses/{secret}",
        "delete",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/beta0/calendars/{calendarId}/secret-addresses/{secret}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/workspace/beta0/calendars/{calendarId}/secret-addresses/{secret}",
        "patch",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/workspace/beta0/calendars/{secret}/calendar.ics", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/calendars/{secret}/calendar.ics", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/config", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/config", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/eu", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/eu", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}/versions", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}/versions", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}/versions/{version}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}/versions/{version}", "put"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}/versions/{version}", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/schemas/{type}/versions/{version}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/us", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/workspace/beta0/us", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/estimates/v1/view-actuals-kpi/annual", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-actuals-kpi/annual", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-actuals-kpi/interim", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-actuals-kpi/interim", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-actuals/annual", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    ("/data/estimates/v1/view-actuals/annual", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-actuals/interim", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    ("/data/estimates/v1/view-actuals/interim", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-summary-kpi/annual", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-summary-kpi/annual", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-summary-kpi/historical-snapshots-kpi", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-summary-kpi/historical-snapshots-kpi", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-summary-kpi/interim", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-summary-kpi/interim", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-summary/annual", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    ("/data/estimates/v1/view-summary/annual", "options"): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-non-periodic-measures",
        "get",
    ): [{"trapi.data.est.sum"}, {"trapi.demo-access", "trapi.data.est.sum"}],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-non-periodic-measures",
        "options",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-periodic-measures-annual",
        "get",
    ): [{"trapi.data.est.sum"}, {"trapi.demo-access", "trapi.data.est.sum"}],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-periodic-measures-annual",
        "options",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-periodic-measures-interim",
        "get",
    ): [{"trapi.data.est.sum"}, {"trapi.demo-access", "trapi.data.est.sum"}],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-periodic-measures-interim",
        "options",
    ): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-summary/historical-snapshots-recommendations", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    (
        "/data/estimates/v1/view-summary/historical-snapshots-recommendations",
        "options",
    ): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-summary/interim", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    ("/data/estimates/v1/view-summary/interim", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/v1/view-summary/non-periodic-measures", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    ("/data/estimates/v1/view-summary/non-periodic-measures", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/v1/view-summary/recommendations", "get"): [
        {"trapi.data.est.sum"},
        {"trapi.demo-access", "trapi.data.est.sum"},
    ],
    ("/data/estimates/v1/view-summary/recommendations", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/iam/mfa-admin/v1/HealthCheck", "get"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/HealthCheck", "options"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Reports", "get"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Reports", "options"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Users", "get"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Users", "options"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Users/account/{locationAccountId}", "get"): [
        {"trapi.platform.iam.mfa"}
    ],
    ("/iam/mfa-admin/v1/Users/account/{locationAccountId}", "options"): [
        {"trapi.platform.iam.mfa"}
    ],
    ("/iam/mfa-admin/v1/Users/{uid}", "get"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Users/{uid}", "put"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/Users/{uid}", "options"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/accounts", "get"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/accounts", "options"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/accounts/{id}", "get"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/accounts/{id}", "put"): [{"trapi.platform.iam.mfa"}],
    ("/iam/mfa-admin/v1/accounts/{id}", "options"): [{"trapi.platform.iam.mfa"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/fund/managed-funds/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/fund/managed-funds/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investment-profile/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investment-profile/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/active/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/active/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/exiting/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/exiting/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/metadata/filter/business-sectors",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/metadata/filter/business-sectors",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/metadata/filter/countries",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/investments/metadata/filter/countries",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/overview/company-information/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/overview/company-information/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/overview/general-information/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/firm/overview/general-information/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/private-company-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/private-company-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/private-company-service/v1/investor/company-overview/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/company-overview/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/contact/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/contact/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/investment-approach/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/investment-approach/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/portfolio-summary/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/portfolio-summary/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/recent-activities/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/recent-activities/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/top-holding/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/investor/top-holding/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/business-classification/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/business-classification/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/corporate-info-ext/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/corporate-info-ext/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/corporate-info/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/corporate-info/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/corporate-structure/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/corporate-structure/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/filings/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/filings/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/financial-performance/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/financial-performance/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/fundamental-ratio/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/fundamental-ratio/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/officer-and-directors/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/officer-and-directors/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/ownership-exists/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/ownership-exists/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/ownership/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/ownership/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/private-equity-investment/round/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/private-equity-investment/round/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/private-equity-investment/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/private-equity-investment/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/related-companies/{id}",
        "get",
    ): [{"trapi.user-framework.workspace.crud"}],
    (
        "/user-framework/mobile/private-company-service/v1/private-company/related-companies/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/iam/technical-product-master/v1/commercial-products", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/commercial-products", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/commercial-products/{productId}", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/commercial-products/{productId}", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/deployment-methods", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/deployment-methods", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/deployment-methods/{id}", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/deployment-methods/{id}", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    (
        "/iam/technical-product-master/v1/deployments/{deploymentId}/permissions",
        "get",
    ): [{"trapi.platform.tech-prod-master"}],
    (
        "/iam/technical-product-master/v1/deployments/{deploymentId}/permissions",
        "options",
    ): [{"trapi.platform.tech-prod-master"}],
    ("/iam/technical-product-master/v1/families", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/families", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/health", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/health", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    (
        "/iam/technical-product-master/v1/product-mappings/content-and-features/searches",
        "post",
    ): [{"trapi.platform.tech-prod-master"}],
    (
        "/iam/technical-product-master/v1/product-mappings/content-and-features/searches",
        "options",
    ): [{"trapi.platform.tech-prod-master"}],
    ("/iam/technical-product-master/v1/products", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/cjrs", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/cjrs", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/deployments", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/deployments", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    (
        "/iam/technical-product-master/v1/products/{productId}/deployments/{deploymentId}",
        "get",
    ): [{"trapi.platform.tech-prod-master"}],
    (
        "/iam/technical-product-master/v1/products/{productId}/deployments/{deploymentId}",
        "options",
    ): [{"trapi.platform.tech-prod-master"}],
    ("/iam/technical-product-master/v1/products/{productId}/permissions", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/permissions", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/ruleset", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/products/{productId}/ruleset", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/rules/evaluation", "post"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/rules/evaluation", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/searches/commercial-products", "post"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/searches/commercial-products", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/searches/technical-products", "post"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/searches/technical-products", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/technical-products", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/technical-products", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/technical-products/{productId}", "get"): [
        {"trapi.platform.tech-prod-master"}
    ],
    ("/iam/technical-product-master/v1/technical-products/{productId}", "options"): [
        {"trapi.platform.tech-prod-master"}
    ],
    (
        "/iam/technical-product-master/v1/technical-products/{productId}/mappings",
        "get",
    ): [{"trapi.platform.tech-prod-master"}],
    (
        "/iam/technical-product-master/v1/technical-products/{productId}/mappings",
        "options",
    ): [{"trapi.platform.tech-prod-master"}],
    ("/iam/role-management/v1/healthcheck", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/healthcheck", "options"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/permission-targets/namespaces", "get"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/permission-targets/namespaces", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/permission-targets/others", "get"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/permission-targets/others", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/permission-targets/types", "get"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/permission-targets/types", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles", "post"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles", "options"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles/{id}", "get"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles/{id}", "delete"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles/{id}", "options"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles/{id}", "patch"): [{"trapi.platform.iam.groups"}],
    ("/iam/role-management/v1/roles/{id}/permissions", "post"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles/{id}/permissions", "delete"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles/{id}/permissions", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles/{id}/users", "get"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles/{id}/users", "post"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles/{id}/users", "delete"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/role-management/v1/roles/{id}/users", "options"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/data-store/v1/graphql", "post"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/v1/graphql", "options"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/v1/graphql/schema/json", "post"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/data-store/v1/graphql/schema/json", "options"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/data-store/v1/graphql/schema/sdl", "get"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/v1/graphql/schema/sdl", "options"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/data-store/v1/info/dependency-status", "get"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/data-store/v1/info/dependency-status", "options"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/data-store/v1/schema/sdl", "get"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/v1/schema/sdl", "options"): [{"trapi.graphql.subscriber.access"}],
    ("/data/ownership/views/ultimate-beneficial-ownership/v1/{dunsNumber}", "get"): [
        {"trapi.data.ownership.ubo.read"}
    ],
    (
        "/data/ownership/views/ultimate-beneficial-ownership/v1/{dunsNumber}",
        "options",
    ): [{"trapi.data.ownership.ubo.read"}],
    ("/user-framework/mobile/alert-service/v1/alerts", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/count", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/count", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/observe", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/observe", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/read", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/read", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/{id}/delete", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/alerts/{id}/delete", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/push-healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/alert-service/v1/push-healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/ecpcdn/beta1/research/docresolver/{documentId}/{format}", "get"): [
        {"trapi.data.research.read"}
    ],
    ("/user-framework/mobile/search-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/search-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/search-service/v1/search", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/search-service/v1/search", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data-management/database-distribution/beta1/operation/transfer-jobs", "get"): [
        {"trapi.transfer-job.ctrl"}
    ],
    (
        "/data-management/database-distribution/beta1/operation/transfer-jobs/cancel/{transferJobId}",
        "delete",
    ): [{"trapi.transfer-job.ctrl"}],
    (
        "/data-management/database-distribution/beta1/operation/transfer-jobs/initiate",
        "post",
    ): [{"trapi.transfer-job.ctrl"}],
    (
        "/data-management/database-distribution/beta1/operation/transfer-jobs/retry/{transferJobId}",
        "put",
    ): [{"trapi.transfer-job.ctrl"}],
    (
        "/data-management/database-distribution/beta1/operation/transfer-jobs/{transferJobId}",
        "get",
    ): [{"trapi.transfer-job.ctrl"}],
    ("/data-management/database-distribution/beta1/operation/user-actions", "get"): [
        {"trapi.transfer-job.ctrl"}
    ],
    ("/data-management/database-distribution/beta1/subscription", "get"): [
        {"trapi.database-subscriptions.crud"}
    ],
    ("/data-management/database-distribution/beta1/subscription", "post"): [
        {"trapi.database-subscriptions.crud"}
    ],
    (
        "/data-management/database-distribution/beta1/subscription/{subscriptionId}",
        "get",
    ): [{"trapi.database-subscriptions.crud"}],
    (
        "/data-management/database-distribution/beta1/subscription/{subscriptionId}",
        "put",
    ): [{"trapi.database-subscriptions.crud"}],
    ("/discovery/auto-suggest/v1/suggest", "get"): [{"trapi.autosuggest.read"}],
    ("/discovery/auto-suggest/v1/suggest", "options"): [{"trapi.autosuggest.read"}],
    ("/search/beta1", "post"): [{"trapi.search.read"}],
    ("/search/beta1", "options"): [{"trapi.search.read"}],
    ("/search/beta1/health", "get"): [{"trapi.search.read"}],
    ("/search/beta1/health", "options"): [{"trapi.search.read"}],
    ("/search/beta1/lookup", "post"): [
        {"trapi.search.read", "trapi.search.lookup.read"}
    ],
    ("/search/beta1/lookup", "options"): [
        {"trapi.search.read", "trapi.search.lookup.read"}
    ],
    ("/search/beta1/metadata/views/{view}", "get"): [
        {"trapi.search.read", "trapi.search.metadata.read"}
    ],
    ("/search/beta1/metadata/views/{view}", "options"): [
        {"trapi.search.read", "trapi.search.metadata.read"}
    ],
    ("/apiadmin/beta0/aggregate/definitions", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/aggregate/definitions", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/aggregate/definitions/{id}", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/aggregate/definitions/{id}", "options"): [
        {"trapi.data.api.test"}
    ],
    ("/apiadmin/beta0/approval", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/approval", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/approval/grant", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/approval/grant", "post"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/approval/require", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/approval/require", "post"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/connectivity", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/connectivity", "post"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/definitions", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/definitions", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/definitions/{id}", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/definitions/{id}", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/definitions/{id}/swagger", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/definitions/{id}/swagger", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/redirect", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/redirect", "options"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/support", "get"): [{"trapi.data.api.test"}],
    ("/apiadmin/beta0/support", "options"): [{"trapi.data.api.test"}],
    ("/user-framework/recently-used/beta1/etp", "get"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/etp", "post"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/etp", "delete"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/etp", "options"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/etp/uuids/{region}", "get"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/etp/uuids/{region}", "delete"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/healthcheck", "get"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/beta1/healthcheck", "options"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/discovery/searchlight/v1", "post"): [{"trapi.searchcore.read"}],
    ("/discovery/searchlight/v1", "options"): [{"trapi.searchcore.read"}],
    ("/discovery/searchlight/v1/health", "get"): [{"trapi.searchcore.read"}],
    ("/discovery/searchlight/v1/health", "options"): [{"trapi.searchcore.read"}],
    ("/discovery/searchlight/v1/lookup", "post"): [
        {"trapi.searchcore.lookup.read", "trapi.searchcore.read"}
    ],
    ("/discovery/searchlight/v1/lookup", "options"): [
        {"trapi.searchcore.lookup.read", "trapi.searchcore.read"}
    ],
    ("/discovery/searchlight/v1/metadata/views/{view}", "get"): [
        {"trapi.searchcore.metadata.read", "trapi.searchcore.read"}
    ],
    ("/discovery/searchlight/v1/metadata/views/{view}", "options"): [
        {"trapi.searchcore.metadata.read", "trapi.searchcore.read"}
    ],
    ("/service-insight/v1/active-alerts", "get"): [{"trapi.serviceinsight.access"}],
    ("/service-insight/v1/active-alerts", "options"): [{"trapi.serviceinsight.access"}],
    ("/service-insight/v1/active-alerts/maintenance-alerts", "get"): [
        {"trapi.serviceinsight.access"}
    ],
    ("/service-insight/v1/active-alerts/maintenance-alerts", "options"): [
        {"trapi.serviceinsight.access"}
    ],
    ("/service-insight/v1/alerts/messages/{messageID}", "get"): [
        {"trapi.serviceinsight.access"}
    ],
    ("/service-insight/v1/alerts/messages/{messageID}", "options"): [
        {"trapi.serviceinsight.access"}
    ],
    ("/service-insight/v1/alerts/{alertID}", "get"): [{"trapi.serviceinsight.access"}],
    ("/service-insight/v1/alerts/{alertID}", "options"): [
        {"trapi.serviceinsight.access"}
    ],
    ("/service-insight/v1/historical-alerts", "get"): [{"trapi.serviceinsight.access"}],
    ("/service-insight/v1/historical-alerts", "options"): [
        {"trapi.serviceinsight.access"}
    ],
    (
        "/data/quantitative-analytics/v1/async-cashflow-currency-conversion-beta",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics/v1/async-cashflow-currency-conversion-beta",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics/v1/async-cashflow-currency-conversion-beta2",
        "post",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics/v1/async-cashflow-currency-conversion-beta2",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics/v1/async-cva", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-cva", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-financial-contracts", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-financial-contracts", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-lifecycle-events", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-lifecycle-events", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-operation/{operationId}", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-operation/{operationId}", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-resource/{operationId}", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-resource/{operationId}", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-synthetic-time-series", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-synthetic-time-series", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-total-return", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/async-total-return", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/cashflow-currency-conversion-beta", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/cashflow-currency-conversion-beta", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/cashflow-currency-conversion-beta2", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/cashflow-currency-conversion-beta2", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/financial-contract-scenarios/beta", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/financial-contract-scenarios/beta", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/financial-contracts", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/financial-contracts", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/financial-contracts-with-solver", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/financial-contracts-with-solver", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/lifecycle-events", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/lifecycle-events", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/metadata-beta/data-mapping", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/metadata-beta/data-mapping", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/metadata-beta/data-mapping", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/metadata-beta/indices/interestrates", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics/v1/metadata-beta/indices/interestrates",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics/v1/metadata-beta/sources/capvolatility", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    (
        "/data/quantitative-analytics/v1/metadata-beta/sources/capvolatility",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics/v1/metadata-beta/sources/swaptionvolatility",
        "get",
    ): [{"trapi.data.quantitative-analytics.read"}],
    (
        "/data/quantitative-analytics/v1/metadata-beta/sources/swaptionvolatility",
        "options",
    ): [{"trapi.data.quantitative-analytics.read"}],
    ("/data/quantitative-analytics/v1/metadata-beta/templates/swaps", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/metadata-beta/templates/swaps", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/synthetic-time-series", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/synthetic-time-series", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/total-return", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/v1/total-return", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/streaming/formula-analytics/v1", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/streaming/formula-analytics/v1", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/data/analytics/benchmark/v1/metadata/calculation-types", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/calculation-types", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/filter-types", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/filter-types", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/related-rics", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/related-rics", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/response-codes", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/response-codes", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/trade-classifications", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/trade-classifications", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/venues", "get"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/metadata/venues", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/request", "options"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/data/analytics/benchmark/v1/request", "post"): [
        {"trapi.data.benchmark.bmk_read"}
    ],
    ("/user-data/lists/beta1/lists", "get"): [],
    ("/user-data/lists/beta1/lists", "post"): [],
    ("/user-data/lists/beta1/lists", "options"): [],
    ("/user-data/lists/beta1/lists/{listID}", "get"): [],
    ("/user-data/lists/beta1/lists/{listID}", "put"): [],
    ("/user-data/lists/beta1/lists/{listID}", "delete"): [],
    ("/user-data/lists/beta1/lists/{listID}", "options"): [],
    ("/user-data/lists/beta1/lists/{listID}/items", "post"): [],
    ("/user-data/lists/beta1/lists/{listID}/items", "options"): [],
    ("/user-data/lists/beta1/lists/{listID}/items/{itemID}", "put"): [],
    ("/user-data/lists/beta1/lists/{listID}/items/{itemID}", "delete"): [],
    ("/user-data/lists/beta1/lists/{listID}/items/{itemID}", "options"): [],
    ("/user-data/lists/beta1/metamodel", "get"): [],
    ("/user-data/lists/beta1/metamodel", "options"): [],
    ("/data-store/beta1/graph-snapshot", "post"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/beta1/graph-snapshot", "options"): [
        {"trapi.graphql.subscriber.access"}
    ],
    ("/data-store/beta1/schema/sdl", "get"): [{"trapi.graphql.subscriber.access"}],
    ("/data-store/beta1/schema/sdl", "options"): [{"trapi.graphql.subscriber.access"}],
    ("/user-data/portfolios/v1/common", "get"): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/common", "options"): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/common/{scope}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}", "put"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}/properties", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}/properties", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}/properties", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}/properties", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}/properties/list", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/common/{scope}/{code}/properties/list", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/enablesearch", "post"): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/enablesearch", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}", "put"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}", "delete"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/commands", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/commands", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/expansion", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/expansion", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/holdings", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/holdings", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/portfolios", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/portfolios", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/portfolios/{portfolioScope}/{portfolioCode}",
        "delete",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/portfolios/{portfolioScope}/{portfolioCode}",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/properties", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/properties", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/properties/$delete",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/properties/$delete",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/properties/$upsert",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/properties/$upsert",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/subgroups", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/subgroups", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/subgroups/{subgroupScope}/{subgroupCode}",
        "delete",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/subgroups/{subgroupScope}/{subgroupCode}",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/transactions", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/transactions",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/transactions/$build",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/portfoliogroups/{scope}/{code}/transactions/$build",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/portfoliosummaries", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliosummaries", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliosummaries/{scope}/{code}", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/portfoliosummaries/{scope}/{code}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/referenceportfolios/{scope}", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/referenceportfolios/{scope}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/referenceportfolios/{scope}/{code}/constituents",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/referenceportfolios/{scope}/{code}/constituents",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/referenceportfolios/{scope}/{code}/constituents",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/referenceportfolios/{scope}/{code}/constituentsadjustments",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/referenceportfolios/{scope}/{code}/constituentsadjustments",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}", "post"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/$tradeticket",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/$tradeticket",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/a2b", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/a2b", "options"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/a2bmovements",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/a2bmovements",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/bucketedCashFlows",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/bucketedCashFlows",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/cashflows",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/cashflows",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/cashladder",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/cashladder",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/cashstatement",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/cashstatement",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdings", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdings", "put"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdings",
        "delete",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdings",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdingsadjustments",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdingsadjustments",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdingstimeseries",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/holdingstimeseries",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/summary", "get"): [
        {"trapi.data.portfolios.read"}
    ],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/summary",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/transactions",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/transactions",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/transactions",
        "delete",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/transactions",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/transactions/$build",
        "post",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/transactions/$build",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/upsertablecashflows",
        "get",
    ): [{"trapi.data.portfolios.read"}],
    (
        "/user-data/portfolios/v1/transactionportfolios/{scope}/{code}/upsertablecashflows",
        "options",
    ): [{"trapi.data.portfolios.read"}],
    ("/platform-information/user-reports/v1/administration", "post"): [
        {"trapi.platform.reports", "trapi.platform.reports.admin"}
    ],
    ("/platform-information/user-reports/v1/administration", "options"): [
        {"trapi.platform.reports"}
    ],
    (
        "/platform-information/user-reports/v1/custom-reports/a00348171/last-login",
        "post",
    ): [{"trapi.platform.reports.lastlogin", "trapi.platform.reports"}],
    (
        "/platform-information/user-reports/v1/custom-reports/a00348171/last-login",
        "options",
    ): [{"trapi.platform.reports"}],
    ("/platform-information/user-reports/v1/info", "get"): [{"trapi.platform.reports"}],
    ("/platform-information/user-reports/v1/info", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/info/{reportCode}/{version}", "get"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/info/{reportCode}/{version}", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/permissions", "post"): [
        {"trapi.platform.reports", "trapi.platform.reports.permissions"}
    ],
    ("/platform-information/user-reports/v1/permissions", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/query-execution-service", "get"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/query-execution-service", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/query-execution-service/{id}", "get"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/query-execution-service/{id}", "delete"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/query-execution-service/{id}", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/reports", "post"): [
        {"trapi.platform.reports", "trapi.platform.reports.usage"}
    ],
    ("/platform-information/user-reports/v1/reports", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/platform-information/user-reports/v1/templates/{reportCode}/versions", "get"): [
        {"trapi.platform.reports"}
    ],
    (
        "/platform-information/user-reports/v1/templates/{reportCode}/versions",
        "options",
    ): [{"trapi.platform.reports"}],
    ("/platform-information/user-reports/v1/usage", "post"): [
        {"trapi.platform.reports", "trapi.platform.reports.usage"}
    ],
    ("/platform-information/user-reports/v1/usage", "options"): [
        {"trapi.platform.reports"}
    ],
    ("/iam/self-registration/v1/profile-registration", "post"): [
        {"trapi.selfregistration.crud"}
    ],
    ("/iam/self-registration/v1/profile-registration", "options"): [
        {"trapi.selfregistration.crud"}
    ],
    ("/alerts/custom-alerts/v1/alert-types", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/alert-types", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/match-events", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/match-events", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/metadata", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/metadata", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/metadata/{alertType}/{collection}", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/metadata/{alertType}/{collection}", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications", "delete"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications", "patch"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/counts/search", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/counts/search", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/counts/unobserved", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/counts/unobserved", "put"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/counts/unobserved", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/counts/unread", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/latest", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/latest", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/search", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/search", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/state", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/notifications/state", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences", "put"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences", "patch"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences/email", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences/email", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences/email/{code}", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/preferences/email/{code}", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions", "post"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions", "delete"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions", "patch"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions/{id}", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions/{id}", "put"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions/{id}", "delete"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions/{id}", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/subscriptions/{id}", "patch"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/universe", "get"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/alerts/custom-alerts/v1/universe", "options"): [
        {
            "trapi.alerts.history.crud",
            "trapi.alerts.subscription.crud",
            "trapi.alerts.preferences.crud",
            "trapi.alerts.publication.crud",
        }
    ],
    ("/data/trade-discovery/publisher-services/v1/upload-location", "get"): [
        {"trapi.frtb.publisher"}
    ],
    ("/data/trade-discovery/publisher-services/v1/upload-location", "options"): [
        {"trapi.frtb.publisher"}
    ],
    ("/data/ownership/views/ultimate-beneficial-ownership/v2/{dunsNumber}", "get"): [
        {"trapi.data.ownership.ubo.read"}
    ],
    (
        "/data/ownership/views/ultimate-beneficial-ownership/v2/{dunsNumber}",
        "options",
    ): [{"trapi.data.ownership.ubo.read"}],
    ("/data/aggregates/v1", "get"): [{"trapi.data.aggregates.read"}],
    ("/data/aggregates/v1", "options"): [{"trapi.data.aggregates.read"}],
    ("/data/aggregates/v1/business-classifications", "get"): [
        {"trapi.data.aggregates.read"}
    ],
    ("/data/aggregates/v1/business-classifications", "options"): [
        {"trapi.data.aggregates.read"}
    ],
    ("/message-services/v1/file-store/subscriptions", "get"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/file-store/subscriptions", "post"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/file-store/subscriptions", "delete"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/file-store/subscriptions", "options"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/file-store/subscriptions/{subscriptionID}", "put"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/file-store/subscriptions/{subscriptionID}", "options"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/news-headlines/subscriptions", "get"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/message-services/v1/news-headlines/subscriptions", "post"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/message-services/v1/news-headlines/subscriptions", "delete"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/message-services/v1/news-headlines/subscriptions", "options"): [
        {"trapi.alerts.news-headlines.crud"},
        {"trapi.alerts.news-headlines-1h.crud"},
        {"trapi.alerts.news-headlines-24h.crud"},
    ],
    ("/message-services/v1/news-stories/subscriptions", "get"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/message-services/v1/news-stories/subscriptions", "post"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/message-services/v1/news-stories/subscriptions", "delete"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/message-services/v1/news-stories/subscriptions", "options"): [
        {"trapi.alerts.news-stories.crud"},
        {"trapi.alerts.news-stories-1h.crud"},
        {"trapi.alerts.news-stories-24h.crud"},
    ],
    ("/message-services/v1/rating/subscriptions", "get"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/rating/subscriptions", "post"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/rating/subscriptions", "delete"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/rating/subscriptions", "options"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/rating/subscriptions/{subscriptionID}", "put"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/rating/subscriptions/{subscriptionID}", "options"): [
        {"trapi.cfs.subscriber.read"}
    ],
    ("/message-services/v1/research/subscriptions", "get"): [
        {"trapi.alerts.research.crud"}
    ],
    ("/message-services/v1/research/subscriptions", "post"): [
        {"trapi.alerts.research.crud"}
    ],
    ("/message-services/v1/research/subscriptions", "delete"): [
        {"trapi.alerts.research.crud"}
    ],
    ("/message-services/v1/research/subscriptions", "options"): [
        {"trapi.alerts.research.crud"}
    ],
    ("/streaming/analytics-price-performance/v1", "get"): [
        {"trapi.streaming.prcperf.read"}
    ],
    ("/streaming/analytics-price-performance/v1", "options"): [],
    (
        "/order-rights-management/product-studio-ii/v1/lov/supplier-services/treeOptions",
        "get",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/lov/supplier-services/treeOptions",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    ("/order-rights-management/product-studio-ii/v1/main-api-health", "get"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/main-api-health", "options"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/multisearch/{assetType}", "post"): [
        {"rdpapi.data.psii.read"}
    ],
    (
        "/order-rights-management/product-studio-ii/v1/multisearch/{assetType}",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    ("/order-rights-management/product-studio-ii/v1/ss/api/change-events", "get"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/ss/api/change-events", "post"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/ss/api/change-events", "options"): [
        {"rdpapi.data.psii.read"}
    ],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/change-events/{changeEventId}",
        "put",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/change-events/{changeEventId}",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services",
        "post",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{id}",
        "put",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{id}",
        "delete",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{id}",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/desired",
        "get",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/desired",
        "put",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/desired",
        "post",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/desired",
        "delete",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/desired",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/bulk",
        "post",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/bulk",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/{vendorRuleId}",
        "put",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/{vendorRuleId}",
        "delete",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/{vendorRuleId}/desired",
        "put",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/{vendorRuleId}/desired",
        "post",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/{vendorRuleId}/desired",
        "delete",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/supplier-services/{supplierServiceId}/vendor-rules/{vendorRuleId}/desired",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/vendor-rules/{vendorRuleId}/desiredId",
        "get",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/ss/api/vendor-rules/{vendorRuleId}/desiredId",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    ("/order-rights-management/product-studio-ii/v1/supplier-services", "post"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/supplier-services", "options"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/supplier-services-health", "get"): [
        {"rdpapi.data.psii.read"}
    ],
    (
        "/order-rights-management/product-studio-ii/v1/supplier-services-health",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    ("/order-rights-management/product-studio-ii/v1/supplier-services/{id}", "get"): [
        {"rdpapi.data.psii.read"}
    ],
    (
        "/order-rights-management/product-studio-ii/v1/supplier-services/{id}",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/supplier-services/{id}/vendor-rules",
        "post",
    ): [{"rdpapi.data.psii.read"}],
    (
        "/order-rights-management/product-studio-ii/v1/supplier-services/{id}/vendor-rules",
        "options",
    ): [{"rdpapi.data.psii.read"}],
    ("/order-rights-management/product-studio-ii/v1/token", "get"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/order-rights-management/product-studio-ii/v1/token", "options"): [
        {"rdpapi.data.psii.read"}
    ],
    ("/data/pricing/snapshots/v1", "get"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/snapshots/v1", "options"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/snapshots/v1", "post"): [{"trapi.data.pricing.read"}],
    ("/streaming/custom-instruments/v1/resource", "get"): [
        {"trapi.streaming.synthetic.read"}
    ],
    ("/streaming/custom-instruments/v1/resource", "options"): [],
    ("/user-framework/mobile/watchlist-service/v1/grid/metadata", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/grid/metadata", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/markets", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/markets", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/markets/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/markets/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-bottom-movers", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/watchlist-service/v1/watchlist-bottom-movers",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-deals", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-deals", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-events", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-events", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-list", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-list", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-metadata", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-metadata", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-raisers-fallers", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/watchlist-service/v1/watchlist-raisers-fallers",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-research", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-research", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-templates", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-templates", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-templates-data", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/watchlist-service/v1/watchlist-templates-data",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-templates/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    (
        "/user-framework/mobile/watchlist-service/v1/watchlist-templates/{id}",
        "options",
    ): [{"trapi.user-framework.workspace.crud"}],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-top-movers", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist-top-movers", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist/{id}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist/{id}", "put"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist/{id}", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/watchlist-service/v1/watchlist/{id}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/iam/permissions/v1/api/healthcheck", "get"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/healthcheck", "options"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/definitions", "get"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/definitions", "post"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/definitions", "options"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/definitions/{id}", "get"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/definitions/{id}", "delete"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/definitions/{id}", "options"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/multiple/sets", "post"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/multiple/sets", "delete"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/multiple/sets", "options"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/sets", "get"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/sets", "post"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/sets", "options"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/sets/{id}", "get"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/sets/{id}", "delete"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/iam/permissions/v1/api/permissions/sets/{id}", "options"): [
        {"trapi.platform.iam.permissions"}
    ],
    ("/data/research/v1/bulk-history", "post"): [{"trapi.data.research.read"}],
    ("/data/research/v1/bulk-history", "options"): [{"trapi.data.research.read"}],
    ("/data/research/v1/bulk-request", "post"): [{"trapi.data.research.read"}],
    ("/data/research/v1/bulk-request", "options"): [{"trapi.data.research.read"}],
    ("/data/research/v1/bulk-request/{requestId}", "get"): [
        {"trapi.data.research.read"}
    ],
    ("/data/research/v1/bulk-request/{requestId}", "put"): [
        {"trapi.data.research.read"}
    ],
    ("/data/research/v1/bulk-request/{requestId}", "options"): [
        {"trapi.data.research.read"}
    ],
    ("/data/research/v1/bulk-request/{requestId}", "patch"): [
        {"trapi.data.research.read"}
    ],
    ("/data/research/v1/bulk-status", "post"): [{"trapi.data.research.read"}],
    ("/data/research/v1/bulk-status", "options"): [{"trapi.data.research.read"}],
    ("/data/research/v1/documents/{documentId}/{format}", "get"): [
        {"trapi.data.research.read"}
    ],
    ("/data/research/v1/documents/{documentId}/{format}", "options"): [
        {"trapi.data.research.read"}
    ],
    ("/iam/replay/notification-messages/v1/api/healthcheck", "get"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/iam/replay/notification-messages/v1/api/replay", "get"): [
        {"trapi.platform.iam.groups"}
    ],
    ("/auth/cloud-credentials/v1", "get"): [{"trapi.auth.cloud-credentials"}],
    ("/auth/cloud-credentials/v1", "options"): [{"trapi.auth.cloud-credentials"}],
    ("/file-store/beta1/buckets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/buckets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/buckets/{name}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/buckets/{name}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/buckets/{name}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/buckets/{name}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/claim-check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/claim-check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}/status", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}/status/{status}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}/status/{status}", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/file-sets/{id}/status/{status}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files/{id}/stream", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/files/{id}/stream", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/beta1/version", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets", "post"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets/{name}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets/{name}", "put"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets/{name}", "delete"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/buckets/{name}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/bulk-publish", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/bulk-publish", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/claim-check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/claim-check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}/status", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}/status", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}/status/{status}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}/status/{status}", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/file-sets/{id}/status/{status}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files/{id}", "put"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files/{id}/stream", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/files/{id}/stream", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/info/health", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/info/health", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/info/s3check", "post"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/info/s3check", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/info/version", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/info/version", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages", "post"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages/{id}", "get"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read", "trapi.cfs.claimcheck.write"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages/{id}", "put"): [
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages/{id}", "delete"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write", "trapi.cfs.publisher.write"},
        {"trapi.cfs.claimcheck.read", "trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.write", "trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages/{id}", "options"): [
        {"trapi.cfs.subscriber.read"},
        {"trapi.cfs.publisher.read"},
        {"trapi.cfs.publisher.write"},
        {"trapi.cfs.publisher.setup.write"},
        {"trapi.cfs.claimcheck.read"},
        {"trapi.cfs.claimcheck.write"},
    ],
    ("/file-store/v1/packages/{id}", "head"): [
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {"trapi.cfs.subscriber.read", "trapi.cfs.publisher.read"},
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.read",
            "trapi.cfs.publisher.write",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.setup.write",
            "trapi.cfs.publisher.read",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.claimcheck.read",
            "trapi.cfs.publisher.read",
        },
        {
            "trapi.cfs.subscriber.read",
            "trapi.cfs.publisher.read",
            "trapi.cfs.claimcheck.write",
        },
    ],
    ("/data/filings/beta1/retrieval/search/{identifier}/{value}", "get"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/filings/beta1/retrieval/search/{identifier}/{value}", "options"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/filings/beta1/retrieval/{filename}", "get"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/filings/beta1/retrieval/{filename}", "options"): [
        {"trapi.data.filings.retrieval"}
    ],
    ("/data/pricing/beta3/snapshots", "get"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/beta3/snapshots", "post"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/beta3/snapshots", "options"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/beta3/views/chains", "get"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/beta3/views/chains", "options"): [{"trapi.data.pricing.read"}],
    ("/data/estimates/beta1/view-actuals-kpi/annual", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-actuals-kpi/annual", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-actuals-kpi/interim", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-actuals-kpi/interim", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-actuals/annual", "get"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-actuals/annual", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-actuals/interim", "get"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-actuals/interim", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-summary-kpi/annual", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary-kpi/annual", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary-kpi/historical-snapshots-kpi", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary-kpi/historical-snapshots-kpi", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary-kpi/interim", "get"): [
        {"trapi.data.estimates.kpi", "trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary-kpi/interim", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary/annual", "get"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-summary/annual", "options"): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-non-periodic-measures",
        "get",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-non-periodic-measures",
        "options",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-periodic-measures-annual",
        "get",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-periodic-measures-annual",
        "options",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-periodic-measures-interim",
        "get",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-periodic-measures-interim",
        "options",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-recommendations",
        "get",
    ): [{"trapi.data.est.sum"}],
    (
        "/data/estimates/beta1/view-summary/historical-snapshots-recommendations",
        "options",
    ): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-summary/interim", "get"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-summary/interim", "options"): [{"trapi.data.est.sum"}],
    ("/data/estimates/beta1/view-summary/non-periodic-measures", "get"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary/non-periodic-measures", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary/recommendations", "get"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/estimates/beta1/view-summary/recommendations", "options"): [
        {"trapi.data.est.sum"}
    ],
    ("/data/pricing/chains/v1", "get"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/chains/v1", "options"): [{"trapi.data.pricing.read"}],
    ("/streaming/benchmark/v1/resource", "get"): [{"trapi.data.benchmark.bmk_read"}],
    ("/streaming/benchmark/v1/resource", "options"): [],
    ("/user-framework/recently-used/v1/etp", "get"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/etp", "post"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/etp", "delete"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/etp", "options"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/etp/uuids/{region}", "get"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/etp/uuids/{region}", "delete"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/healthcheck", "get"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/user-framework/recently-used/v1/healthcheck", "options"): [
        {"trapi.user-framework.recently-used.crud"}
    ],
    ("/data/wealth-clients/v1/groups", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/groups", "post"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/groups", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/groups/{scope}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}", "put"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}", "delete"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/balances", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/balances", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/groups/{scope}/{code}/historical-valuation-summary",
        "get",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/groups/{scope}/{code}/historical-valuation-summary",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/holdings", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/holdings", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/orders", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/orders", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/portfolio", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/portfolio", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/groups/{scope}/{code}/portfolio/{portfolioScope}/{portfolioCode}",
        "delete",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/groups/{scope}/{code}/portfolio/{portfolioScope}/{portfolioCode}",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/portfolios", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/portfolios", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/portfolios/$delete", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/portfolios/$delete", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/refresh/portfolios", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/refresh/portfolios", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/taxlots/closed", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/taxlots/closed", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/taxlots/open", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/taxlots/open", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/transactions", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/transactions", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/valuation", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/valuation", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/valuation-summary", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/groups/{scope}/{code}/valuation-summary", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/instruments", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/instruments", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/legal-entities", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/legal-entities", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/legal-entities/{scope}/{type}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/legal-entities/{scope}/{type}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/navigation", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/navigation", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/news/headlines", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/news/headlines", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/news/{storyId}/story", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/news/{storyId}/story", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages/{id}", "delete"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages/{id}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages/{id}/status", "put"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-action-messages/{id}/status", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/next-best-actions", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}", "delete"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata", "patch"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata/{metadataKey}",
        "get",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata/{metadataKey}",
        "put",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata/{metadataKey}",
        "delete",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/next-best-actions/{scope}/{code}/metadata/{metadataKey}",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}/status", "put"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/next-best-actions/{scope}/{code}/status", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/next-best-actions/{scope}/{code}/status/history",
        "get",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/next-best-actions/{scope}/{code}/status/history",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/orders", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/orders", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/persons", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/persons", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/persons/{scope}/{type}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/persons/{scope}/{type}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/persons/{scope}/{type}/{code}/plaid/add-institution",
        "post",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/persons/{scope}/{type}/{code}/plaid/add-institution",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/persons/{scope}/{type}/{code}/plaid/link-token",
        "post",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/persons/{scope}/{type}/{code}/plaid/link-token",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/persons/{scope}/{type}/{code}/plaid/remove-institution",
        "post",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/persons/{scope}/{type}/{code}/plaid/remove-institution",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/portfolios", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/portfolios", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/balances", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/balances", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/portfolios/{scope}/{code}/historical-valuation-summary",
        "get",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/portfolios/{scope}/{code}/historical-valuation-summary",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/holdings", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/holdings", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/orders", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/orders", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/taxlots/closed", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/taxlots/closed", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/taxlots/open", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/taxlots/open", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/transactions", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/transactions", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/valuation", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/valuation", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/portfolios/{scope}/{code}/valuation-summary", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/portfolios/{scope}/{code}/valuation-summary",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/relationshipdefinitions", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/relationshipdefinitions", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/relationshipdefinitions", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}", "put"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}", "delete"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    (
        "/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}/relationships",
        "post",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}/relationships",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}/relationships/$delete",
        "post",
    ): [{"trapi.wealth.clientdata"}],
    (
        "/data/wealth-clients/v1/relationshipdefinitions/{scope}/{code}/relationships/$delete",
        "options",
    ): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/groups", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/groups", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/legal-entities", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/search/legal-entities", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/search/persons", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/persons", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/search/portfolios", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/portfolios", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/search/properties", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/properties", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/search/relations", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/search/relations", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/sequences", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/sequences", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/sequences/{scope}", "post"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/sequences/{scope}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/sequences/{scope}/{code}", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/sequences/{scope}/{code}", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/sequences/{scope}/{code}/next", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/sequences/{scope}/{code}/next", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/transactions", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/transactions", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/valuation", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/valuation", "options"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/valuation-summary", "get"): [{"trapi.wealth.clientdata"}],
    ("/data/wealth-clients/v1/valuation-summary", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/valuation-summary/groups", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/valuation-summary/groups", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/valuation-summary/portfolios", "get"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/wealth-clients/v1/valuation-summary/portfolios", "options"): [
        {"trapi.wealth.clientdata"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/add-periods", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/add-periods", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/c3", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/c3", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/count-periods", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/count-periods", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/date-schedule", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/date-schedule", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/holidays", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/holidays", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/holidays", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/is-working-day", "get"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/is-working-day", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics-dates-and-calendars/v1/is-working-day", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/streaming/benchmark/beta1/resource", "get"): [{"trapi.data.benchmark.bmk_read"}],
    ("/streaming/benchmark/beta1/resource", "options"): [],
    ("/data/pricing/quality-of-service/v1", "get"): [{"trapi.data.pricing.read"}],
    ("/data/pricing/quality-of-service/v1", "options"): [{"trapi.data.pricing.read"}],
    ("/data/hvmi/v1/approvals/{masterKey}", "get"): [],
    ("/data/hvmi/v1/approvals/{masterKey}", "options"): [],
    ("/data/hvmi/v1/approvals/{masterKey}/{transactionId}", "get"): [],
    ("/data/hvmi/v1/approvals/{masterKey}/{transactionId}", "post"): [],
    ("/data/hvmi/v1/approvals/{masterKey}/{transactionId}", "options"): [],
    ("/data/hvmi/v1/changelog/conviction-lists/{listId}", "get"): [],
    ("/data/hvmi/v1/changelog/conviction-lists/{listId}", "options"): [],
    ("/data/hvmi/v1/changelog/convictions/{convictionId}", "get"): [],
    ("/data/hvmi/v1/changelog/convictions/{convictionId}", "options"): [],
    ("/data/hvmi/v1/changelog/documents/{documentId}", "get"): [],
    ("/data/hvmi/v1/changelog/documents/{documentId}", "options"): [],
    ("/data/hvmi/v1/changelog/house-views/{houseViewId}", "get"): [],
    ("/data/hvmi/v1/changelog/house-views/{houseViewId}", "options"): [],
    ("/data/hvmi/v1/changelog/portfolios/{portfolioId}", "get"): [],
    ("/data/hvmi/v1/changelog/portfolios/{portfolioId}", "options"): [],
    ("/data/hvmi/v1/comments/{moduleName}", "get"): [],
    ("/data/hvmi/v1/comments/{moduleName}", "options"): [],
    ("/data/hvmi/v1/comments/{moduleName}/{itemId}", "get"): [],
    ("/data/hvmi/v1/comments/{moduleName}/{itemId}", "options"): [],
    ("/data/hvmi/v1/conviction-lists", "get"): [],
    ("/data/hvmi/v1/conviction-lists", "options"): [],
    ("/data/hvmi/v1/conviction-lists-page", "get"): [],
    ("/data/hvmi/v1/conviction-lists-page", "options"): [],
    ("/data/hvmi/v1/conviction-lists/{listId}", "get"): [],
    ("/data/hvmi/v1/conviction-lists/{listId}", "options"): [],
    ("/data/hvmi/v1/convictions", "get"): [],
    ("/data/hvmi/v1/convictions", "put"): [],
    ("/data/hvmi/v1/convictions", "delete"): [],
    ("/data/hvmi/v1/convictions", "options"): [],
    ("/data/hvmi/v1/convictions-page", "get"): [],
    ("/data/hvmi/v1/convictions-page", "options"): [],
    ("/data/hvmi/v1/convictions/import", "post"): [],
    ("/data/hvmi/v1/convictions/import/batch/{Id}", "get"): [],
    ("/data/hvmi/v1/convictions/import/batch/{Id}", "options"): [],
    ("/data/hvmi/v1/convictions/{convictionId}", "get"): [],
    ("/data/hvmi/v1/convictions/{convictionId}", "options"): [],
    ("/data/hvmi/v1/dashboard", "get"): [],
    ("/data/hvmi/v1/dashboard", "options"): [],
    ("/data/hvmi/v1/documents", "get"): [],
    ("/data/hvmi/v1/documents", "options"): [],
    ("/data/hvmi/v1/documents-page", "get"): [],
    ("/data/hvmi/v1/documents-page", "options"): [],
    ("/data/hvmi/v1/documents/download/{documentId}", "get"): [],
    ("/data/hvmi/v1/documents/download/{documentId}", "options"): [],
    ("/data/hvmi/v1/documents/{documentId}", "get"): [],
    ("/data/hvmi/v1/documents/{documentId}", "options"): [],
    ("/data/hvmi/v1/documentsimporter", "post"): [],
    ("/data/hvmi/v1/documentsimporter/batch/{Id}", "get"): [],
    ("/data/hvmi/v1/documentsimporter/batch/{Id}", "options"): [],
    ("/data/hvmi/v1/fileUpload", "post"): [],
    ("/data/hvmi/v1/fileUpload", "options"): [],
    ("/data/hvmi/v1/health", "get"): [],
    ("/data/hvmi/v1/health", "post"): [],
    ("/data/hvmi/v1/health", "options"): [],
    ("/data/hvmi/v1/house-views", "get"): [],
    ("/data/hvmi/v1/house-views", "options"): [],
    ("/data/hvmi/v1/house-views-page", "get"): [],
    ("/data/hvmi/v1/house-views-page", "options"): [],
    ("/data/hvmi/v1/house-views/{houseViewId}", "get"): [],
    ("/data/hvmi/v1/house-views/{houseViewId}", "options"): [],
    ("/data/hvmi/v1/master", "get"): [],
    ("/data/hvmi/v1/master", "options"): [],
    ("/data/hvmi/v1/merge/{uniqueId}", "get"): [],
    ("/data/hvmi/v1/merge/{uniqueId}", "post"): [],
    ("/data/hvmi/v1/merge/{uniqueId}", "options"): [],
    ("/data/hvmi/v1/portfolios", "get"): [],
    ("/data/hvmi/v1/portfolios", "options"): [],
    ("/data/hvmi/v1/portfolios-page", "get"): [],
    ("/data/hvmi/v1/portfolios-page", "options"): [],
    ("/data/hvmi/v1/portfolios/{portfolioId}", "get"): [],
    ("/data/hvmi/v1/portfolios/{portfolioId}", "options"): [],
    ("/data/hvmi/v1/quicksearch/{searchText}", "get"): [],
    ("/data/hvmi/v1/quicksearch/{searchText}", "options"): [],
    ("/data/hvmi/v1/search", "get"): [],
    ("/data/hvmi/v1/search", "options"): [],
    ("/data/hvmi/v1/summary/{masterKey}/{transactionId}", "get"): [],
    ("/data/hvmi/v1/summary/{masterKey}/{transactionId}", "options"): [],
    ("/data/quantitative-analytics/beta1/financial-contracts", "post"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/quantitative-analytics/beta1/financial-contracts", "options"): [
        {"trapi.data.quantitative-analytics.read"}
    ],
    ("/data/news/v1/analyze", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/analyze", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/headlines", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/headlines", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/images/{imageId}", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/images/{imageId}", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/metadata", "get"): [
        {"trapi.metadata.read"},
        {"trapi.data.news.read"},
    ],
    ("/data/news/v1/metadata", "options"): [
        {"trapi.metadata.read"},
        {"trapi.data.news.read"},
    ],
    ("/data/news/v1/metadata/{id}", "get"): [
        {"trapi.metadata.read"},
        {"trapi.data.news.read"},
    ],
    ("/data/news/v1/metadata/{id}", "options"): [
        {"trapi.metadata.read"},
        {"trapi.data.news.read"},
    ],
    ("/data/news/v1/metadata/{id}/children", "get"): [
        {"trapi.metadata.read"},
        {"trapi.data.news.read"},
    ],
    ("/data/news/v1/metadata/{id}/children", "options"): [
        {"trapi.metadata.read"},
        {"trapi.data.news.read"},
    ],
    ("/data/news/v1/online-reports", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/online-reports", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/online-reports/{reportId}", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/online-reports/{reportId}", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/stories/{storyId}", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/stories/{storyId}", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/suggest", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/suggest", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/top-news", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/top-news", "options"): [{"trapi.data.news.read"}],
    ("/data/news/v1/top-news/{topNewsId}", "get"): [{"trapi.data.news.read"}],
    ("/data/news/v1/top-news/{topNewsId}", "options"): [{"trapi.data.news.read"}],
    ("/iam/resources/v1/api/healthcheck", "get"): [{"trapi.platform.iam.resources"}],
    ("/iam/resources/v1/api/healthcheck", "options"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions", "get"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions", "post"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions", "options"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions/{id}", "get"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions/{id}", "put"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions/{id}", "delete"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/definitions/{id}", "options"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances", "get"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances", "post"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances", "options"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances/{id}", "get"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances/{id}", "put"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances/{id}", "delete"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/iam/resources/v1/api/resources/instances/{id}", "options"): [
        {"trapi.platform.iam.resources"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations/{version}", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations/{version}", "put"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations/{version}", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/configurations/{version}", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/healthcheck", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/healthcheck", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/settings/follow", "get"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/settings/follow", "post"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/settings/follow", "delete"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/settings/follow", "options"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/mobile/config-service/v1/settings/follow", "patch"): [
        {"trapi.user-framework.workspace.crud"}
    ],
    ("/user-framework/auctions/v1/administration/create-client", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/create-user", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/create-user-profile", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/edit-user", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/edit-user-notifications", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-clients-for-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-clients-list", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-countries", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/get-deleted-users-by-current-user",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/administration/get-deleted-users-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/get-file-structure-data", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/get-permission-groups-for-user",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/get-permissions-for-profile", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/get-permissions-selector-for-profile",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/administration/get-profile-permissions-audit-dates-by-profile-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/administration/get-required-info-for-create-external-user-by-client-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/get-user-audits-for-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-user-by-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-user-desks-selector", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-user-info-for-edit-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/get-user-notifications-by-uuid",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/administration/get-user-permission-for-profile-model",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/administration/get-user-profile-permission-audits-by-date-and-profile",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/get-user-search-results", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-users-by-current-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/get-users-for-client", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/get-users-profiles-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/administration/get-users-profiles-for-client-by-group",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/search-admin-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/search-client-admin-by-eikon-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/search-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/search-user-by-eikon-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/set-user-instance", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/soft-delete-user", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/soft-delete-user-profile", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/un-delete-user", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/update-file-structure-data", "put"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/administration/update-permissions-with-profile",
        "put",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/administration/update-user-desks", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/administration/update-user-profile", "put"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/bidding-administration/create-bidding-organisation",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/bidding-administration/create-clearing", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/bidding-administration/create-desk-for-bidding-organisation",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/bidding-administration/create-investor", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/bidding-administration/create-settlement-account",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/bidding-administration/delete-clearing", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/bidding-administration/delete-investor", "delete"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/bidding-administration/edit-settlement-account",
        "put",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-available-desks-for-settlement-account",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-bidding-organisation-broker-editability-status",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-bidding-organisations-and-investors",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-bidding-organisations-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-bidding-organisations-issues-types-by-client-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-bidding-organisations-maps-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-clearing-for-edit-by-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-clearings-by-client-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-desks-for-bidding-organisation",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-investors-by-bidding-organisation-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-light-investors-by-bidding-organisation-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-light-investors-for-current-user",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-non-broker-bidding-organisations-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-only-investors-by-bidding-organisation-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-settlement-account-desks-selector",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/get-settlement-accounts-for-investor",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/soft-delete-bidding-organisation",
        "delete",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/soft-delete-bidding-organisation-used-for-issue",
        "delete",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/soft-delete-desk-for-bidding-organisation",
        "delete",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/soft-delete-settlement-account",
        "delete",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/bidding-administration/update-bidding-organisation",
        "put",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/bidding-administration/update-clearing", "put"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/bidding-administration/update-desk-for-bidding-organisation",
        "put",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/bidding-administration/update-investor", "put"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/bidding-administration/update-settlement-account-desks",
        "put",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/convert-html-to-pdf", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/get-auction-list-field-mappings", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/get-client-currency-details", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/dashboard/get-client-issue-sub-type-precision-category-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/dashboard/get-client-issue-types-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/dashboard/get-currencies", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/dashboard/get-customization-model-for-client",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/dashboard/get-issue-sub-types-by-client-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/get-preset-mappings", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/dashboard/get-preset-mappings-for-issue-type",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/dashboard/get-presets-default-models", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/get-presets-for-client", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/dashboard/process-client-issue-sub-type-precision-category-mappings",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/dashboard/process-default-for-preset", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/save-preset", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/submit-auction-list", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/update-client-currency-details", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/dashboard/update-issue-types-for-client", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/export/get-files-by-schema", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/health/liveness-probe", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/health/readiness-probe", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/identity-api-proxy", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/apply-scenario", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/break-allocation-lock", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/calculate-dates-based-on-settlement", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/calculate-elapsed-days", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/calculate-issue-dates", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/cancel-auction", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/cancel-auction-template", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/change-issue-status", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/create-issue-template", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/create-new-auction", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/create-template-from-issue", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/delete-issue", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/delete-issue-template", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/edit-auction", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-archived-issues", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-auction-composite-edit-model", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-auction-event-date-time", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-audits-for-issue", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-bond-structure", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-current-issues", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-deal-by-issue-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-by-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-by-super-and-issue-name", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-composite-model", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-statuses", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-statuses-for-trade-book", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-statuses-model", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-sub-type-composite-model", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issue-templates-by-client", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-issues-for-trade-book", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-pending-issues", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-recent-issues", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-refreshed-data", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-scenario-names", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-scenarios-by-issue-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-super-auction-bond-tenor-lists", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-super-auction-details", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-super-auction-tenor-lists", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/get-template-details", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/issue-has-order-pending-for-import", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/mark-trade-as-received", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/save-scenario", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/send-deal", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/send-trades-for-auction", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/set-allocation-lock", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/submit-template", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/try-change-issue-status", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/update-issue-template", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/issue/update-scenario-status", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/adjudicate-bdl", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/allow-order", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/approve-order", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/calculate-price-yield", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/cancel-order", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/check-if-user-can-bid-on-auction", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/create-order", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/create-orders-pending-for-import", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/delete-order-pending-for-import", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/delete-orders-pending-for-import", "delete"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/download-bid-template", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/edit-order", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/export-allocations", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/export-allocations-all", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/export-allocations-json", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/export-allocationsbids", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/export-analysis-data", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/export-to-excel", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/get-allocated-conventional-auctions",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/orders/get-allocation-dto-for-auction", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-allocation-dto-for-issue", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-allocation-dtos", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-allocation-summary", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-allocation-summary-auction", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-analysis-data", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-audits-for-order", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-bdl-allocation-model", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/get-bdl-allocation-model-for-published-auction",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/orders/get-bdl-allocation-model-for-scenario",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/orders/get-bdl-allocation-summary", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-bdl-allocation-summary-auction", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-bid-summary", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-biddings-for-open-issue", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/get-cancelled-orders-for-closed-auction-with-max-allocation-time",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/orders/get-deal-ticket", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/get-investor-bids-allocation-model-by-issue-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/orders/get-investor-bids-for-user-by-auction-id",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/orders/get-issue-statistics-for-bidding-organisations",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/orders/get-issue-statistics-for-external", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/get-order-allocation-view-dto-for-bid-org",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/orders/get-order-chart-data-item-models-for-issue",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/orders/get-orders-for-issue", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/get-orders-for-issue-all", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/get-pending-for-import-orders-for-issue",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/orders/get-primary-bidder-bdl-allocation-model-for-published-auction",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/orders/get-scenario-report", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/import-orders", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/make-investor-bid", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/make-investor-bids", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/process-allocation", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/process-investor-bids-allocations", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/recalculate-ypr", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/refuse-order", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/reject-order", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/save-allocation", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/save-bdl-allocation", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/save-published-orders", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/save-published-scenario", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/orders/try-save-allocation", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/orders/update-orders-bids-for-allocation-import",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/query-trades", "post"): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/search-user", "post"): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/sendmail", "post"): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/settings/get-clients-with-issue-sub-types", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/get-settings", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/get-settings-by-client-id", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/get-user-detail", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/get-user-settings", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/get-version", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/health-check", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/health-check-detailed", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/save-user-settings", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/submit-dashboard-settings", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/settings/submit-settings", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/solace/query-trades", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/solace/update-status", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/tracs-process-bus-cal", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/tracs-process-frb-calculate", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/tracs-process-frb-calculate-many", "post"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/trading-insights/basic-analytics/export-to-excel",
        "get",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/trading-insights/basic-analytics/export-to-excel-token",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/trading-insights/basic-analytics/get-basic-analytics",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/trading-insights/basic-analytics/get-dashboard-lists",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/trading-insights/basic-analytics/get-dashboard-sub-type-filter",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/trading-insights/basic-analytics/get-fetch-auction-filters",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/trading-insights/currency/pairs", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/trading-insights/custom-report/export-to-excel-token",
        "get",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/trading-insights/custom-report/list", "get"): [
        {"trapi.trading.auctions"}
    ],
    (
        "/user-framework/auctions/v1/trading-insights/custom-report/run-default",
        "post",
    ): [{"trapi.trading.auctions"}],
    (
        "/user-framework/auctions/v1/trading-insights/custom-report/run-with-filters",
        "post",
    ): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/trading-insights/date/ranges", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/trading-insights/session", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/trading-insights/session/client", "put"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/trading-insights/session/client/list", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/trading-insights/userinfo", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/trading-insights/versioninfo", "get"): [
        {"trapi.trading.auctions"}
    ],
    ("/user-framework/auctions/v1/update-status", "post"): [{"trapi.trading.auctions"}],
    ("/user-framework/auctions/v1/upload-to-event-platform-ftp", "post"): [
        {"trapi.trading.auctions"}
    ],
    ("/data/trading-analytics/trade-data/beta1/redi/history", "get"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
    ("/data/trading-analytics/trade-data/beta1/redi/history", "post"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
    ("/data/trading-analytics/trade-data/beta1/redi/history", "options"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
    ("/data/trading-analytics/trade-data/beta1/reference/ric-info", "get"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
    ("/data/trading-analytics/trade-data/beta1/reference/ric-info", "options"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
    ("/data/trading-analytics/trade-data/beta1/statistics/orders-count", "get"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
    ("/data/trading-analytics/trade-data/beta1/statistics/orders-count", "options"): [
        {
            "trapi.data.trading-analytics.tds.report.redi",
            "trapi.data.trading-analytics.tds.history.redi",
        }
    ],
}
